@@ -44,8 +44,9 @@ clear_all()
 	}
 	for (i = 0; var_names[i]; i++) {
 		free(var_names[i]);
+		var_names[i] = NULL;
 	}
-	var_names[0] = NULL;
+	clear_sign_array();
 }
 
 /*
@@ -78,6 +79,14 @@ init_gvars()
 	symb_flag = false;
 	sign_flag = false;
 	in_calc_cmd = false;
+
+	zero_token.level = 1;
+	zero_token.kind = CONSTANT;
+	zero_token.token.constant = 0.0;
+
+	one_token.level = 1;
+	one_token.kind = CONSTANT;
+	one_token.token.constant = 1.0;
 }
 
 /*
@@ -87,12 +96,24 @@ init_gvars()
 clean_up()
 {
 	init_gvars();
-#if	!SECURE
 	if (gfp != stdout) {
+#if	!SECURE
 		fclose(gfp);
+#endif
 		gfp = stdout;
 	}
-#endif
+}
+
+/*
+ * Restart sign array.
+ */
+clear_sign_array()
+{
+	int	i;
+
+	for (i = 0; i < ARR_CNT(sign_array); i++) {
+		sign_array[i] = false;
+	}
 }
 
 /*
@@ -102,8 +123,7 @@ set_sign_array()
 {
 	int	i, j;
 
-	for (i = 0; i < ARR_CNT(sign_array); i++)
-		sign_array[i] = false;
+	clear_sign_array();
 	for (i = 0; i < n_equations; i++) {
 		if (n_lhs[i] > 0) {
 			for (j = 0; j < n_lhs[i]; j += 2) {
@@ -211,8 +231,7 @@ int	i;
 {
 	if (n_rhs[i] <= 0)
 		return false;
-	if (n_lhs[i] != 1 || lhs[i][0].kind != VARIABLE
-	    || (lhs[i][0].token.variable & VAR_MASK) <= SIGN)
+	if (n_lhs[i] != 1 || lhs[i][0].kind != VARIABLE || (lhs[i][0].token.variable & VAR_MASK) <= SIGN)
 		return false;
 	if (found_var(rhs[i], n_rhs[i], lhs[i][0].token.variable))
 		return false;
@@ -256,13 +275,13 @@ long	v;
 }
 
 /*
- * Substitute every instance of "v" in "equation" with "exp".
+ * Substitute every instance of "v" in "equation" with "expression".
  */
-subst_var_with_exp(equation, np, exp, len, v)
-token_type	*equation;
-int		*np;		/* pointer to equation length */
-token_type	*exp;
-int		len;		/* exp length */
+subst_var_with_exp(equation, np, expression, len, v)
+token_type	*equation;	/* equation side pointer */
+int		*np;		/* pointer to equation side length */
+token_type	*expression;	/* expression pointer */
+int		len;		/* expression length */
 long		v;
 {
 	int	j, k;
@@ -276,7 +295,7 @@ long		v;
 			}
 			blt(&equation[j+len], &equation[j+1], (*np - (j + 1)) * sizeof(token_type));
 			*np += len - 1;
-			blt(&equation[j], exp, len * sizeof(token_type));
+			blt(&equation[j], expression, len * sizeof(token_type));
 			for (k = j; k < j + len; k++)
 				equation[k].level += level;
 		}
@@ -289,13 +308,17 @@ long		v;
 int
 min_level(equation, n)
 token_type	*equation;	/* expression pointer */
-int		n;		/* expression size */
+int		n;		/* expression length */
 {
 	int		min1;
 	token_type	*p1, *ep;
 
 	if (n <= 1) {
-		return equation[0].level;
+		if (n <= 0) {
+			return 1;
+		} else {
+			return equation[0].level;
+		}
 	}
 	min1 = equation[1].level;
 	ep = &equation[n];
@@ -335,7 +358,7 @@ char	*cp;
 		if (extra_garbage(cp))
 			return -1;
 	}
-	if (notdefined(i)) {
+	if (not_defined(i)) {
 		return -1;
 	}
 	return i;
@@ -349,7 +372,7 @@ char	*cp;
 int
 parse_expr(equation, np, cp)
 token_type	*equation;	/* where the parsed expression is stored */
-int		*np;		/* pointer to parsed expression size */
+int		*np;		/* pointer to parsed expression length */
 char		*cp;		/* string to parse */
 {
 	int	i;
@@ -394,16 +417,16 @@ token_type	*equation;
 int		*np;
 {
 	char	buf[10000];
-	char	*cp1;
+	char	*cp;
 
 	for (;;) {
-		if ((cp1 = getstring(buf, sizeof(buf))) == NULL) {
+		if ((cp = get_string(buf, sizeof(buf))) == NULL) {
 			return false;
 		}
 		if (!case_sensitive_flag) {
-			str_tolower(cp1);
+			str_tolower(cp);
 		}
-		if (parse_expr(equation, np, cp1))
+		if (parse_expr(equation, np, cp))
 			break;
 	}
 	return(*np > 0);
@@ -419,17 +442,17 @@ prompt_var(vp)
 long	*vp;
 {
 	char	buf[MAX_CMD_LEN];
-	char	*cp1;
+	char	*cp;
 
 prompt_again:
 	my_strlcpy(prompt_str, _("Enter variable: "), sizeof(prompt_str));
-	if ((cp1 = getstring(buf, sizeof(buf))) == NULL) {
+	if ((cp = get_string(buf, sizeof(buf))) == NULL) {
 		return false;
 	}
-	cp1 = parse_var2(vp, cp1);
-	if (cp1 == NULL)
+	cp = parse_var2(vp, cp);
+	if (cp == NULL)
 		return false;
-	if (*cp1) {
+	if (*cp) {
 		error(_("Only one variable may be specified."));
 		goto prompt_again;
 	}
@@ -440,11 +463,31 @@ prompt_again:
  * Return true and display a message if equation "i" is undefined.
  */
 int
-notdefined(i)
+not_defined(i)
 int	i;	/* equation space number */
 {
+	if (i < 0 || i >= n_equations) {
+		error(_("Invalid equation number."));
+		return true;
+	}
+	if (n_lhs[i] <= 0) {
+		error(_("Equation space empty."));
+		return true;
+	}
+	return false;
+}
+
+/*
+ * Return true and display a message if current equation is undefined.
+ */
+int
+current_not_defined()
+{
+	int	i;
+
+	i = cur_equation;
 	if (i < 0 || i >= n_equations || n_lhs[i] <= 0) {
-		error(_("Equation not defined."));
+		error(_("No current equation or expression."));
 		return true;
 	}
 	return false;
@@ -459,14 +502,17 @@ int	i;	/* equation space number */
  * Returns "string" if successful or NULL on error.
  */
 char	*
-getstring(string, n)
+get_string(string, n)
 char	*string;	/* storage for input string */
 int	n;		/* maximum size of "string" in bytes */
 {
 #if	LIBRARY
+	error(_("Missing command line argument."));
 	return NULL;
 #else
+#if	READLINE
 	char	*cp;
+#endif
 
 	if (quiet_mode) {
 		prompt_str[0] = '\0';	/* don't display a prompt */
@@ -506,15 +552,15 @@ int	n;		/* maximum size of "string" in bytes */
 int
 get_yes_no()
 {
-	char	*cp1;
+	char	*cp;
 	char	buf[MAX_CMD_LEN];
 
 	for (;;) {
-		if ((cp1 = getstring(buf, sizeof(buf))) == NULL) {
+		if ((cp = get_string(buf, sizeof(buf))) == NULL) {
 			return false;
 		}
-		str_tolower(cp1);
-		switch (*cp1) {
+		str_tolower(cp);
+		switch (*cp) {
 		case 'n':
 			return false;
 		case 'y':
@@ -527,10 +573,14 @@ get_yes_no()
  * Store or display the result of a command.
  */
 return_result(en)
-int	en;
+int	en;	/* equation number */
 {
 #if	LIBRARY
 	if (gfp == stdout) {
+		if (factor_int_flag) {
+			factor_int(lhs[en], &n_lhs[en]);
+			factor_int(rhs[en], &n_rhs[en]);
+		}
 		result_str = list_equation(en, false);
 		return;
 	}
@@ -549,7 +599,8 @@ char	*cp;
 }
 
 /*
- * Process an equation number range given on the command line.
+ * Process an equation number range given in text string *cpp.
+ * Skip past all spaces and update *cpp to point to the next argument.
  *
  * Return true if successful,
  * with the starting equation number in "*ip"
@@ -569,7 +620,7 @@ int	*ip, *jp;
 	} else {
 		i = cur_equation;
 	}
-	if (notdefined(i)) {
+	if (not_defined(i)) {
 		return false;
 	}
 	*ip = i;
@@ -582,7 +633,7 @@ int	*ip, *jp;
 		*jp = n_equations - 1;
 	} else {
 		if (isascii(**cpp) && isdigit(**cpp)) {
-			*ip = decstrtol(*cpp, cpp) - 1;
+			*ip = strtol(*cpp, cpp, 10) - 1;
 		} else {
 			*ip = cur_equation;
 		}
@@ -591,16 +642,16 @@ int	*ip, *jp;
 			return false;
 		}
 		if (**cpp != '-') {
-			if (notdefined(*ip)) {
+			if (not_defined(*ip)) {
 				return false;
 			}
 			*jp = *ip;
+			*cpp = skip_space(*cpp);
 			return true;
 		}
 		(*cpp)++;
-		*cpp = skip_space(*cpp);
 		if (isascii(**cpp) && isdigit(**cpp)) {
-			*jp = decstrtol(*cpp, cpp) - 1;
+			*jp = strtol(*cpp, cpp, 10) - 1;
 		} else {
 			*jp = cur_equation;
 		}
@@ -614,6 +665,7 @@ int	*ip, *jp;
 			*jp = i;
 		}
 	}
+	*cpp = skip_space(*cpp);
 	for (i = *ip; i <= *jp; i++) {
 		if (n_lhs[i] > 0) {
 			return true;
@@ -627,10 +679,14 @@ int	*ip, *jp;
 /*
  * This function is provided to make sure there is nothing else
  * on a command line.
+ *
+ * Returns false if OK.
+ * Returns true if non-space characters were encountered
+ * and an error message was printed.
  */
 int
 extra_garbage(cp)
-char	*cp;
+char	*cp;	/* command line */
 {
 	cp = skip_space(cp);
 	if (*cp) {
@@ -641,7 +697,7 @@ char	*cp;
 }
 
 /*
- * get_range() if it is the last option on the command line.
+ * get_range() if it is the last possible option on the command line.
  */
 int
 get_range_eol(cpp, ip, jp)
@@ -687,7 +743,10 @@ char	*cp, **cpp;
 }
 
 /*
- * Skip over a parameter (command line argument).
+ * Skip over a parameter (command line argument) in a character string.
+ * Parameters are separated with spaces.
+ *
+ * Returns a string pointer to the next parameter.
  */
 char	*
 skip_param(cp)
@@ -728,7 +787,7 @@ char	*cp1, *cp2;
 int
 level_plus(p1, n1, level)
 token_type	*p1;	/* expression pointer */
-int		n1;	/* expression size */
+int		n1;	/* expression length */
 int		level;	/* parentheses level number */
 {
 	int	i;
@@ -753,7 +812,7 @@ int		level;	/* parentheses level number */
 int
 level1_plus(p1, n1)
 token_type	*p1;	/* expression pointer */
-int		n1;	/* expression size */
+int		n1;	/* expression length */
 {
 	int	level;
 
@@ -767,7 +826,7 @@ int		n1;	/* expression size */
 int
 var_count(p1, n1)
 token_type	*p1;	/* expression pointer */
-int		n1;	/* expression size */
+int		n1;	/* expression length */
 {
 	int	i;
 	int	count;
@@ -780,3 +839,21 @@ int		n1;	/* expression size */
 	}
 	return count;
 }
+
+/*
+ * Return true if expression contains infinity or nan.
+ */
+int
+exp_contains_infinity(p1, n1)
+token_type	*p1;	/* expression pointer */
+int		n1;	/* expression length */
+{
+	int	i;
+
+	for (i = 0; i < n1; i += 2) {
+		if (p1[i].kind == CONSTANT && !isfinite(p1[i].token.constant)) {
+			return true;
+		}
+	}
+	return false;
+}
