@@ -6,37 +6,44 @@
 
 #include "includes.h"
 
-#define	OPT_MIN_SIZE	7	/* Minimum size of repeated expressions to find in "optimize" command. */
+#define	OPT_MIN_SIZE	7	/* Minimum size of repeated expressions to find in optimize command. */
 
 static int	sum_product();
 static int	find_more();
 static int	opt_es();
 static int	complex_func();
-static int	compare_rhs();
 static int	elim_sub();
 static int	edit_sub();
 
+/* Global variables for the optimize command. */
 static int	opt_en[N_EQUATIONS];
 static int	last_temp_var = 0;
 
+#if	!LIBRARY
 /*
  * The version command.
  */
 int
 version_cmd(cp)
-char	*cp;
+char	*cp;	/* the command line argument */
 {
-	if (extra_garbage(cp))
+	if (extra_garbage(cp)) {
+		/* if anything on the command line, fail */
 		return false;
-#if	LIBRARY
-	if (gfp == stdout) {
-		result_str = VERSION;
-		return true;
 	}
-	fprintf(gfp, "%s\n", VERSION);
-	return true;
-#else
+	return version_report();
+}
+
+/*
+ * Report version number and compile flags.
+ */
+int
+version_report()
+{
 	fprintf(gfp, _("Mathomatic version %s\nCompile flags used: "), VERSION);
+#if	__STDC__
+	fprintf(gfp, "__STDC__ ");
+#endif
 #if	UNIX
 	fprintf(gfp, "UNIX ");
 #endif
@@ -63,8 +70,8 @@ char	*cp;
 #endif
 	fprintf(gfp, "\n");
 	return true;
-#endif
 }
+#endif
 
 /*
  * The sum command.
@@ -86,24 +93,24 @@ char	*cp;
 	return sum_product(cp, true);
 }
 
+/*
+ * Common function for the sum and product commands.
+ */
 static int
 sum_product(cp, product_flag)
-char	*cp;
-int	product_flag;
+char	*cp;		/* the command line */
+int	product_flag;	/* true for product, otherwise sum */
 {
 	int		i, j;
-	long		v;
-	char		*cp1;
-	double		start, end, step;
+	long		v = 0;
+	double		start, end, step = 1.0;
 	int		result_en;
 	int		n, ns;
 	token_type	*dest, *source;
-	int		count_down;
-	char		buf[MAX_CMD_LEN];
+	int		count_down;		/* if true, count down, otherwise count up */
+	char		*cp1, buf[MAX_CMD_LEN];
 
-	v = 0L;
-	step = 1.0;
-	if (notdefined(cur_equation)) {
+	if (current_not_defined()) {
 		return false;
 	}
 	result_en = next_espace();
@@ -117,13 +124,13 @@ int	product_flag;
 		dest = lhs[result_en];
 	}
 	if (*cp == '\0') {
-		for (j = 0; j < ns; j++) {
+		for (j = 0; j < ns; j += 2) {
 			if (source[j].kind == VARIABLE) {
 				if ((source[j].token.variable & VAR_MASK) <= SIGN)
 					continue;
 				if (v) {
 					if (v != source[j].token.variable) {
-						v = 0L;
+						v = 0;
 						break;
 					}
 				} else {
@@ -131,7 +138,7 @@ int	product_flag;
 				}
 			}
 		}
-		if (v == 0L) {
+		if (v == 0) {
 			if (!prompt_var(&v))
 				return false;
 		}
@@ -148,9 +155,9 @@ int	product_flag;
 	if (*cp) {
 		cp1 = cp;
 	} else {
-		list_var(v, false);
+		list_var(v, 0);
 		snprintf(prompt_str, sizeof(prompt_str), "%s = ", var_str);
-		if ((cp1 = getstring(buf, sizeof(buf))) == NULL)
+		if ((cp1 = get_string(buf, sizeof(buf))) == NULL)
 			return false;
 	}
 	start = strtod(cp1, &cp);
@@ -163,7 +170,7 @@ int	product_flag;
 		cp1 = cp;
 	} else {
 		my_strlcpy(prompt_str, _("To: "), sizeof(prompt_str));
-		if ((cp1 = getstring(buf, sizeof(buf))) == NULL)
+		if ((cp1 = get_string(buf, sizeof(buf))) == NULL)
 			return false;
 	}
 	end = strtod(cp1, &cp);
@@ -174,7 +181,7 @@ int	product_flag;
 	cp = skip_space(cp);
 	if (*cp) {
 		cp1 = cp;
-		step = strtod(cp1, &cp);
+		step = fabs(strtod(cp1, &cp));
 		if (step <= 0.0 || step >= MAX_K_INTEGER || cp1 == cp) {
 			error(_("Invalid step."));
 			return false;
@@ -186,12 +193,10 @@ int	product_flag;
 	if (fmod(fabs(start - end) / step, 1.0)) {
 		printf(_("Warning: end value not reached.\n"));
 	}
-	dest[0].kind = CONSTANT;
-	dest[0].level = 1;
 	if (product_flag) {
-		dest[0].token.constant = 1.0;
+		dest[0] = one_token;
 	} else {
-		dest[0].token.constant = 0.0;
+		dest[0] = zero_token;
 	}
 	n = 1;
 	for (; count_down ? (start >= end) : (start <= end); count_down ? (start -= step) : (start += step)) {
@@ -235,7 +240,7 @@ int	product_flag;
 	return true;
 }
 
-#if	!BASICS
+#if	!BASICS && !LIBRARY
 /*
  * This function is for the "optimize" command.
  * It finds and substitutes all occurrences of "en"
@@ -257,37 +262,34 @@ int		en;		/* equation number */
 	found_se = true;
 	for (level = 1; found_se; level++) {
 		for (i = 1, found_se = false; i < *np; i = j + 2) {
-			for (j = i; j < *np && equation[j].level > level; j += 2) {
-			}
+			for (j = i; j < *np && equation[j].level > level; j += 2)
+				;
 			if (j == i) {
 				continue;
 			}
 			found_se = true;
 			k = i - 1;
-			if ((j - k) >= OPT_MIN_SIZE
-			    && se_compare(&equation[k], j - k, rhs[en], n_rhs[en], &diff_sign)) {
+			if ((j - k) >= OPT_MIN_SIZE && se_compare(&equation[k], j - k, rhs[en], n_rhs[en], &diff_sign)) {
 				v = lhs[en][0].token.variable;
 				if (diff_sign) {
 					blt(&equation[i+2], &equation[j], (*np - j) * sizeof(token_type));
 					*np -= (j - (i + 2));
-					equation[k].level = level + 1;
+					level++;
+					equation[k].level = level;
 					equation[k].kind = CONSTANT;
 					equation[k].token.constant = -1.0;
 					k++;
-					equation[k].level = level + 1;
+					equation[k].level = level;
 					equation[k].kind = OPERATOR;
 					equation[k].token.operatr = TIMES;
 					k++;
-					equation[k].level = level + 1;
-					equation[k].kind = VARIABLE;
-					equation[k].token.variable = v;
 				} else {
 					blt(&equation[i], &equation[j], (*np - j) * sizeof(token_type));
 					*np -= (j - i);
-					equation[k].level = level;
-					equation[k].kind = VARIABLE;
-					equation[k].token.variable = v;
 				}
+				equation[k].level = level;
+				equation[k].kind = VARIABLE;
+				equation[k].token.variable = v;
 				return true;
 			}
 		}
@@ -308,8 +310,7 @@ token_type	*equation;
 int		*np;
 {
 	int	i, j, k;
-	int	i1, j1, k1;
-	int	i2;
+	int	i1, i2, jj1, k1;
 	int	level, level1;
 	int	diff_sign;
 	int	found_se, found_se1;
@@ -318,8 +319,8 @@ int		*np;
 	found_se = true;
 	for (level = 1; found_se; level++) {
 		for (i = 1, found_se = false; i < *np; i = j + 2) {
-			for (j = i; j < *np && equation[j].level > level; j += 2) {
-			}
+			for (j = i; j < *np && equation[j].level > level; j += 2)
+				;
 			if (j == i) {
 				continue;
 			}
@@ -330,18 +331,18 @@ int		*np;
 			}
 			found_se1 = true;
 			for (level1 = 1; found_se1; level1++) {
-				for (i1 = 1, found_se1 = false; i1 < *np; i1 = j1 + 2) {
-					for (j1 = i1; j1 < *np && equation[j1].level > level1; j1 += 2) {
+				for (i1 = 1, found_se1 = false; i1 < *np; i1 = jj1 + 2) {
+					for (jj1 = i1; jj1 < *np && equation[jj1].level > level1; jj1 += 2) {
 					}
-					if (j1 == i1) {
+					if (jj1 == i1) {
 						continue;
 					}
 					found_se1 = true;
 					if (i1 <= j)
 						continue;
 					k1 = i1 - 1;
-					if ((j1 - k1) >= OPT_MIN_SIZE
-					    && se_compare(&equation[k], j - k, &equation[k1], j1 - k1, &diff_sign)) {
+					if ((jj1 - k1) >= OPT_MIN_SIZE
+					    && se_compare(&equation[k], j - k, &equation[k1], jj1 - k1, &diff_sign)) {
 						if (last_temp_var > MAX_SUBSCRIPT) {
 							last_temp_var = 0;
 						}
@@ -355,26 +356,24 @@ int		*np;
 						blt(rhs[i2], &equation[k], (j - k) * sizeof(token_type));
 						n_rhs[i2] = j - k;
 						if (diff_sign) {
-							blt(&equation[i1+2], &equation[j1], (*np - j1) * sizeof(token_type));
-							*np -= (j1 - (i1 + 2));
-							equation[k1].level = level1 + 1;
+							blt(&equation[i1+2], &equation[jj1], (*np - jj1) * sizeof(token_type));
+							*np -= (jj1 - (i1 + 2));
+							level1++;
+							equation[k1].level = level1;
 							equation[k1].kind = CONSTANT;
 							equation[k1].token.constant = -1.0;
 							k1++;
-							equation[k1].level = level1 + 1;
+							equation[k1].level = level1;
 							equation[k1].kind = OPERATOR;
 							equation[k1].token.operatr = TIMES;
 							k1++;
-							equation[k1].level = level1 + 1;
-							equation[k1].kind = VARIABLE;
-							equation[k1].token.variable = v;
 						} else {
-							blt(&equation[i1], &equation[j1], (*np - j1) * sizeof(token_type));
-							*np -= (j1 - i1);
-							equation[k1].level = level1;
-							equation[k1].kind = VARIABLE;
-							equation[k1].token.variable = v;
+							blt(&equation[i1], &equation[jj1], (*np - jj1) * sizeof(token_type));
+							*np -= (jj1 - i1);
 						}
+						equation[k1].level = level1;
+						equation[k1].kind = VARIABLE;
+						equation[k1].token.variable = v;
 						blt(&equation[i], &equation[j], (*np - j) * sizeof(token_type));
 						*np -= j - i;
 						equation[k].level = level;
@@ -407,9 +406,9 @@ char	*cp;
 	int	i, j, k;
 	int	start, stop;
 	int	i1;
-	int	rv, flag;
+	int	rv = false;
+	int	flag;
 
-	rv = false;
 	if (!get_range_eol(&cp, &start, &stop)) {
 		return false;
 	}
@@ -460,12 +459,15 @@ int
 push_cmd(cp)
 char	*cp;
 {
-	int	i;
+	int	i, j;
 
-	if ((i = get_default_en(cp)) < 0)
+	if (!get_range_eol(&cp, &i, &j)) {
 		return false;
-	push_en(i);
-	debug_string(0, _("Equation pushed.  Hit the UP key to access."));
+	}
+	for (; i <= j; i++) {
+		push_en(i);
+	}
+	debug_string(0, _("Expression(s) pushed.  Press the UP key to access."));
 	return true;
 }
 
@@ -473,7 +475,7 @@ char	*cp;
  * Push an equation space into the readline history.
  */
 push_en(en)
-int	en;
+int	en;	/* equation space number */
 {
 	high_prec = true;
 	add_history(list_equation(en, false));
@@ -491,50 +493,15 @@ char	*cp;
 	char	buf[MAX_CMD_LEN];
 
 	if (*cp == '\0') {
-		printf(_("Options are set as follows:\n\n"));
-#if	!SILENT
-		printf("debug_level = %d\n", debug_level);
-#endif
-		if (!case_sensitive_flag) {
-			printf("no ");
-		}
-		printf("case_sensitive\n");
+		fprintf(gfp, _("Options are set as follows:\n\n"));
 
-		if (!color_flag) {
-			printf("no ");
-		}
-		printf("color\n");
+		output_options();
 
-		if (!bold_colors) {
-			printf("no ");
-		}
-		printf("bold_colors\n");
-
-		printf("columns = %d\n", screen_columns);
-
-		if (!groupall) {
-			printf("no ");
-		}
-		printf("display2d\n");
-
-		if (!preserve_roots) {
-			printf("no ");
-		}
-		printf("preserve_roots\n");
-
-		if (!true_modulus) {
-			printf("no ");
-		}
-		printf("true_modulus\n");
-
-		if (!finance_option) {
-			printf("no ");
-		}
-		printf("finance\n");
+		fprintf(gfp, "columns = %d\n", screen_columns);
 
 #if	!SECURE
 		if (getcwd(buf, sizeof(buf))) {
-			printf("directory = %s\n", buf);
+			fprintf(gfp, "directory = %s\n", buf);
 		}
 #endif
 		return true;
@@ -542,8 +509,61 @@ char	*cp;
 	return set_options(cp);
 }
 
+/*
+ * Output the current set options in a format suitable for RC_FILE.
+ */
+output_options()
+{
+#if	!SILENT
+	fprintf(gfp, "debug_level = %d\n", debug_level);
+#endif
+
+	if (!case_sensitive_flag) {
+		fprintf(gfp, "no ");
+	}
+	fprintf(gfp, "case_sensitive\n");
+
+	if (!color_flag) {
+		fprintf(gfp, "no ");
+	}
+	fprintf(gfp, "color\n");
+
+	if (!bold_colors) {
+		fprintf(gfp, "no ");
+	}
+	fprintf(gfp, "bold_colors\n");
+
+	if (!groupall) {
+		fprintf(gfp, "no ");
+	}
+	fprintf(gfp, "display2d\n");
+
+	if (!preserve_roots) {
+		fprintf(gfp, "no ");
+	}
+	fprintf(gfp, "preserve_roots\n");
+
+	if (!true_modulus) {
+		fprintf(gfp, "no ");
+	}
+	fprintf(gfp, "true_modulus\n");
+
+	if (!finance_option) {
+		fprintf(gfp, "no ");
+	}
+	fprintf(gfp, "finance\n");
+
+	if (!factor_int_flag) {
+		fprintf(gfp, "no ");
+	}
+	fprintf(gfp, "factor_integers\n");
+}
+
+/*
+ * Return true if "cp" points to a negative word.
+ */
 int
-isno(cp)
+is_no(cp)
 char	*cp;
 {
 	if (strcmp_tospace(cp, "no") == 0
@@ -555,7 +575,7 @@ char	*cp;
 }
 
 /*
- * Handle parsing of options for "set" command.
+ * Handle parsing of options for the set command.
  *
  * Return false if error.
  */
@@ -565,11 +585,12 @@ char	*cp;
 {
 	int	negate;
 
+next_option:
 	cp = skip_space(cp);
 	if (*cp == '\0') {
 		return true;
 	}
-	negate = isno(cp);
+	negate = is_no(cp);
 	if (negate) {
 		cp = skip_param(cp);
 	}
@@ -586,7 +607,6 @@ char	*cp;
 #if	!SILENT
 	if (strncasecmp(cp, "debug", 5) == 0) {
 		cp = skip_param(cp);
-		negate = (negate || isno(cp));
 		if (negate) {
 			debug_level = 0;
 		} else {
@@ -594,14 +614,13 @@ char	*cp;
 				error(_("Please specify a debug level number."));
 				return false;
 			}
-			debug_level = atoi(cp);
+			debug_level = decstrtol(cp, &cp);
 		}
-		return true;
+		goto next_option;
 	}
 #endif
 	if (strncasecmp(cp, "columns", 7) == 0) {
 		cp = skip_param(cp);
-		negate = (negate || isno(cp));
 		if (negate) {
 			screen_columns = 0;
 		} else {
@@ -609,53 +628,51 @@ char	*cp;
 				error(_("Please specify how wide the screen is in columns."));
 				return false;
 			}
-			screen_columns = atoi(cp);
+			screen_columns = decstrtol(cp, &cp);
 		}
-		return true;
+		goto next_option;
 	}
 	if (strncasecmp(cp, "case", 4) == 0) {
 		cp = skip_param(cp);
-		negate = (negate || isno(cp));
 		case_sensitive_flag = !negate;
-		return true;
+		goto next_option;
 	}
-	if (strncasecmp(cp, "display2d", 7) == 0) {
+	if (strncasecmp(cp, "display2d", 9) == 0) {
 		cp = skip_param(cp);
-		negate = (negate || isno(cp));
 		groupall = !negate;
-		return true;
+		goto next_option;
 	}
 	if (strncasecmp(cp, "preserve", 8) == 0) {
 		cp = skip_param(cp);
-		negate = (negate || isno(cp));
 		preserve_roots = !negate;
-		return true;
+		goto next_option;
 	}
 	if (strncasecmp(cp, "true", 4) == 0) {
 		cp = skip_param(cp);
-		negate = (negate || isno(cp));
 		true_modulus = !negate;
-		return true;
+		goto next_option;
 	}
 	if (strncasecmp(cp, "color", 5) == 0) {
 		cp = skip_param(cp);
-		negate = (negate || isno(cp));
 		color_flag = !negate;
 		cur_color = -1;
-		return true;
+		goto next_option;
 	}
 	if (strncasecmp(cp, "bold", 4) == 0) {
 		cp = skip_param(cp);
-		negate = (negate || isno(cp));
 		bold_colors = !negate;
 		cur_color = -1;
-		return true;
+		goto next_option;
 	}
 	if (strncasecmp(cp, "finance", 7) == 0) {
 		cp = skip_param(cp);
-		negate = (negate || isno(cp));
 		finance_option = !negate;
-		return true;
+		goto next_option;
+	}
+	if (strncasecmp(cp, "factor", 6) == 0) {
+		cp = skip_param(cp);
+		factor_int_flag = !negate;
+		goto next_option;
 	}
 	error(_("Unknown set option."));
 	return false;
@@ -675,12 +692,12 @@ char	*cp;
 		cp = _("Please Press Enter");
 	}
 	snprintf(prompt_str, sizeof(prompt_str), " ==== %s ==== ", cp);
-	if ((cp1 = getstring(buf, sizeof(buf))) == NULL) {
+	if ((cp1 = get_string(buf, sizeof(buf))) == NULL) {
 		return false;
 	}
-	if (strncasecmp(cp1, "quit", 4) == 0
-	    || strncasecmp(cp1, "abort", 5) == 0)
+	if (strncasecmp(cp1, "quit", 4) == 0) {
 		return false;
+	}
 	return true;
 }
 
@@ -719,10 +736,13 @@ char	*cp;
 	return true;
 }
 
+/*
+ * Common function for the imaginary and real commands.
+ */
 static int
 complex_func(cp, imag_flag)
-char	*cp;
-int	imag_flag;
+char	*cp;		/* the command line */
+int	imag_flag;	/* if true, copy the imaginary part, otherwise copy the real part */
 {
 	int		i, j, k;
 	int		beg;
@@ -730,9 +750,12 @@ int	imag_flag;
 	int		has_imag, has_real;
 	token_type	*source, *dest;
 	int		n1, *nps, *np;
+	long		v;
 
-	if ((i = get_default_en(cp)) < 0)
+	if (current_not_defined()) {
 		return false;
+	}
+	i = cur_equation;
 	j = next_espace();
 	if (n_rhs[i]) {
 		source = rhs[i];
@@ -745,16 +768,23 @@ int	imag_flag;
 		dest = lhs[j];
 		np = &n_lhs[j];
 	}
+	v = IMAGINARY;
+	if (*cp) {
+		cp = parse_var2(&v, cp);
+		if (cp == NULL) {
+			return false;
+		}
+		if (extra_garbage(cp))
+			return false;
+	}
 	simp_loop(source, nps);
 	uf_simp(source, nps);
-	factorv(source, nps, IMAGINARY);
+	factorv(source, nps, v);
 	partial_flag = false;
 	uf_simp(source, nps);
 	partial_flag = true;
 	n1 = 1;
-	dest[0].level = 1;
-	dest[0].kind = CONSTANT;
-	dest[0].token.constant = 0.0;
+	dest[0] = zero_token;
 	has_imag = false;
 	has_real = false;
 	k = 0;
@@ -767,7 +797,7 @@ int	imag_flag;
 			    && (source[k].token.operatr == PLUS || source[k].token.operatr == MINUS)) {
 				break;
 			}
-			if (source[k].kind == VARIABLE && source[k].token.variable == IMAGINARY) {
+			if (source[k].kind == VARIABLE && source[k].token.variable == v) {
 				found_imag = true;
 			}
 		}
@@ -783,12 +813,8 @@ int	imag_flag;
 			n1 += (k - beg);
 		}
 	}
-	if (!has_imag) {
-		error(_("No imaginary parts found.  Expression is not complex."));
-		return false;
-	}
-	if (!has_real) {
-		error(_("No real parts found.  Expression is not complex."));
+	if (!has_imag || !has_real) {
+		error(_("Failed, expression is not a mix."));
 		return false;
 	}
 	simp_divide(dest, &n1);
@@ -822,24 +848,6 @@ char	*cp;
 	return complex_func(cp, true);
 }
 
-/*
- * Simplify and approximate for the calculate command.
- * Includes imaginary number simplification.
- */
-calc_simp(equation, np)
-token_type	*equation;
-int		*np;
-{
-	in_calc_cmd = true;
-	subst_constants(equation, np);
-	simp_side(equation, np);
-	uf_simp(equation, np);
-	factorv(equation, np, IMAGINARY);
-	simp_side(equation, np);
-	uf_simp(equation, np);
-	in_calc_cmd = false;
-}
-
 #if	!LIBRARY
 /*
  * The tally command.
@@ -853,9 +861,7 @@ char	*cp;
 
 	if (extra_garbage(cp))
 		return false;
-	trhs[0].kind = CONSTANT;
-	trhs[0].level = 1;
-	trhs[0].token.constant = 0.0;
+	trhs[0] = zero_token;
 	n_trhs = 1;
 	for (count = 0.0;; count++) {
 		fprintf(gfp, _("Running total = "));
@@ -920,20 +926,19 @@ calculate_cmd(cp)
 char	*cp;
 {
 	int		i, j, k;
-	long		v, last_v, it_v;
+	long		v, last_v, it_v = 0;
 	long		counter;
 	long		counter_max;
 	sign_array_type	sa_mark;
 	sign_array_type	sa_value;
-	int		iterations;
+	int		iterations = 1;
 	int		n;
 	token_type	*source;
 
-	iterations = 1;
-	it_v = 0;
-	i = cur_equation;
-	if (notdefined(i))
+	if (current_not_defined()) {
 		return false;
+	}
+	i = cur_equation;
 	if (n_rhs[i]) {
 		source = rhs[i];
 		n = n_rhs[i];
@@ -976,7 +981,7 @@ char	*cp;
 		if ((v & VAR_MASK) <= SIGN || v == it_v) {
 			continue;
 		}
-		list_var(v, false);
+		list_var(v, 0);
 		snprintf(prompt_str, sizeof(prompt_str), _("Enter %s: "), var_str);
 		if (!get_expr(tlhs, &n_tlhs)) {
 			continue;
@@ -990,7 +995,7 @@ char	*cp;
 		if (trhs[j].kind == VARIABLE && trhs[j].token.variable < 0)
 			trhs[j].token.variable = -trhs[j].token.variable;
 	if (it_v) {
-		list_var(it_v, false);
+		list_var(it_v, 0);
 		snprintf(prompt_str, sizeof(prompt_str), _("Enter initial %s: "), var_str);
 		while (!get_expr(scratch, &n))
 			;
@@ -1010,8 +1015,7 @@ char	*cp;
 	for (j = 0; j < ARR_CNT(sa_mark); j++)
 		sa_mark[j] = false;
 	for (j = 0; j < n_trhs; j += 2) {
-		if (trhs[j].kind == VARIABLE
-		    && (trhs[j].token.variable & VAR_MASK) == SIGN) {
+		if (trhs[j].kind == VARIABLE && (trhs[j].token.variable & VAR_MASK) == SIGN) {
 			sa_mark[(trhs[j].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK] = true;
 		}
 	}
@@ -1032,8 +1036,7 @@ char	*cp;
 			}
 		}
 		for (j = 0; j < n_tlhs; j += 2) {
-			if (tlhs[j].kind == VARIABLE
-			    && (tlhs[j].token.variable & VAR_MASK) == SIGN) {
+			if (tlhs[j].kind == VARIABLE && (tlhs[j].token.variable & VAR_MASK) == SIGN) {
 				if (sa_value[(tlhs[j].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK]) {
 					tlhs[j].kind = CONSTANT;
 					tlhs[j].token.constant = -1.0;
@@ -1045,12 +1048,12 @@ char	*cp;
 		}
 		for (j = 0, k = false; j < ARR_CNT(sa_mark); j++) {
 			if (sa_mark[j]) {
-				if (k)
+				if (k) {
 					fprintf(gfp, ", ");
-				else {
+				} else {
 					fprintf(gfp, _("Solution #%ld with "), counter + 1);
 				}
-				list_var((long) SIGN + (((long) j) << VAR_SHIFT), false);
+				list_var((long) SIGN + (((long) j) << VAR_SHIFT), 0);
 				fprintf(gfp, "%s = ", var_str);
 				if (sa_value[j]) {
 					fprintf(gfp, "-1");
@@ -1063,6 +1066,9 @@ char	*cp;
 		if (k)
 			fprintf(gfp, ":\n");
 		calc_simp(tlhs, &n_tlhs);
+		if (factor_int_flag) {
+			factor_int(tlhs, &n_tlhs);
+		}
 		fprintf(gfp, " ");
 		if (n_rhs[i]) {
 			list_proc(lhs[i], n_lhs[i], false);
@@ -1083,18 +1089,27 @@ clear_cmd(cp)
 char	*cp;
 {
 	int	i, j;
+	char	*cp1;
 
-	if (is_all(cp)) {
-		clear_all();
-	} else {
-		if (!get_range_eol(&cp, &i, &j)) {
-			return false;
+	do {
+		cp1 = cp;
+		if (is_all(cp)) {
+			clear_all();
+			return true;
+		} else {
+			if (!get_range(&cp, &i, &j)) {
+				return false;
+			}
+			if (*cp && cp == cp1) {
+				error(_("Invalid argument."));
+				return false;
+			}
+			for (; i <= j; i++) {
+				n_lhs[i] = 0;
+				n_rhs[i] = 0;
+			}
 		}
-		for (; i <= j; i++) {
-			n_lhs[i] = 0;
-			n_rhs[i] = 0;
-		}
-	}
+	} while (*cp);
 	return true;
 }
 
@@ -1142,12 +1157,11 @@ compare_cmd(cp)
 char	*cp;
 {
 	int		i, j;
-	token_type	want;
 	int		diff_sign;
 	int		already_solved;
 
 	i = decstrtol(cp, &cp) - 1;
-	if (notdefined(i)) {
+	if (not_defined(i)) {
 		return false;
 	}
 	if (strcmp_tospace(cp, "with") == 0) {
@@ -1172,8 +1186,8 @@ char	*cp;
 				return true;
 			}
 			debug_string(0, _("Simplifying both expressions..."));
-			simpa_side(lhs[i], &n_lhs[i], false, true);
-			simpa_side(lhs[j], &n_lhs[j], false, true);
+			simpa_side(lhs[i], &n_lhs[i], true);
+			simpa_side(lhs[j], &n_lhs[j], true);
 #if	!SILENT
 			if (debug_level >= 0) {
 				list_sub(i);
@@ -1204,8 +1218,8 @@ char	*cp;
 			goto times_neg1;
 		}
 		debug_string(0, _("Simplifying both equations..."));
-		simpa_side(rhs[i], &n_rhs[i], false, true);
-		simpa_side(rhs[j], &n_rhs[j], false, true);
+		simpa_side(rhs[i], &n_rhs[i], true);
+		simpa_side(rhs[j], &n_rhs[j], true);
 #if	!SILENT
 		if (debug_level >= 0) {
 			list_sub(i);
@@ -1222,11 +1236,8 @@ char	*cp;
 		}
 	}
 	debug_string(0, _("Solving both equations for zero and unfactoring..."));
-	want.level = 1;
-	want.kind = CONSTANT;
-	want.token.constant = 0.0;
-	if (!solve_sub(&want, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i])
-	    || !solve_sub(&want, 1, lhs[j], &n_lhs[j], rhs[j], &n_rhs[j])) {
+	if (solve_sub(&zero_token, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i]) <= 0
+	    || solve_sub(&zero_token, 1, lhs[j], &n_lhs[j], rhs[j], &n_rhs[j]) <= 0) {
 		error(_("Can't solve for zero!"));
 		return false;
 	}
@@ -1237,14 +1248,14 @@ char	*cp;
 		return true;
 	}
 	debug_string(0, _("Simplifying both equations..."));
-	simpa_side(rhs[i], &n_rhs[i], false, true);
-	simpa_side(rhs[j], &n_rhs[j], false, true);
+	simpa_side(rhs[i], &n_rhs[i], true);
+	simpa_side(rhs[j], &n_rhs[j], true);
 	if (compare_rhs(i, j, &diff_sign)) {
 		fprintf(gfp, _("Equations are identical.\n"));
 		return true;
 	}
-	if (!solve_sub(&want, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i])
-	    || !solve_sub(&want, 1, lhs[j], &n_lhs[j], rhs[j], &n_rhs[j])) {
+	if (solve_sub(&zero_token, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i]) <= 0
+	    || solve_sub(&zero_token, 1, lhs[j], &n_lhs[j], rhs[j], &n_rhs[j]) <= 0) {
 		error(_("Can't solve for zero!"));
 		return false;
 	}
@@ -1283,8 +1294,7 @@ char	*cp;
 	long		v, v_tmp;
 	int		i, j;
 	int		nl, nr;
-	double		d1, d2;
-	double		d3, d4;
+	double		d1, d2, d3, d4;
 	complexs	c1, c2, c3;
 
 	v = 0;
@@ -1337,7 +1347,7 @@ char	*cp;
 	if (poly_div(rhs[i], nr, lhs[i], nl, &v_tmp)) {
 		simp_divide(tlhs, &n_tlhs);
 		simp_divide(trhs, &n_trhs);
-		list_var(v_tmp, false);
+		list_var(v_tmp, 0);
 		fprintf(gfp, _("Polynomial division successful using base variable (%s).\n"), var_str);
 		fprintf(gfp, _("The quotient is:\n"));
 		list_proc(tlhs, n_tlhs, false);
@@ -1371,20 +1381,15 @@ int
 eliminate_cmd(cp)
 char	*cp;
 {
-	long	v, last_v, v1;
-	int	i;
-	int	n;
-	int	using_flag;
+	long	v, last_v, v1, va[MAX_VARS];
+	int	i, n;
+	int	using_flag, did_something = false;
 	char	used[N_EQUATIONS];
-	int	did_something;
-	int	vc;
-	long	va[MAX_VARS];
+	int	vc = 0;
 
-	did_something = false;
-	vc = 0;
 	for (i = 0; i < ARR_CNT(used); i++)
 		used[i] = false;
-	if (notdefined(cur_equation)) {
+	if (current_not_defined()) {
 		return false;
 	}
 	if (is_all(cp)) {
@@ -1432,19 +1437,23 @@ next_var:
 	} else {
 		if (did_something) {
 			return_result(cur_equation);
+		} else {
+			error(_("No substitutions made."));
 		}
 		return did_something;
 	}
 	if (!var_in_equation(cur_equation, v)) {
-		list_var(v, false);
+#if	!SILENT
+		list_var(v, 0);
 		printf(_("Variable (%s) not found in current equation.\n"), var_str);
+#endif
 		goto next_var;
 	}
 	using_flag = (strcmp_tospace(cp, "using") == 0);
 	if (using_flag) {
 		cp = skip_param(cp);
 		i = decstrtol(cp, &cp) - 1;
-		if (notdefined(i)) {
+		if (not_defined(i)) {
 			return false;
 		}
 		if (i == cur_equation) {
@@ -1458,8 +1467,6 @@ next_var:
 		i = cur_equation;
 		for (;; n++) {
 			if (n >= n_equations) {
-				list_var(v, false);
-				printf(_("Variable (%s) not found in any other equation.\n"), var_str);
 				goto next_var;
 			}
 			if (i <= 0)
@@ -1489,14 +1496,14 @@ long	v;
 
 #if	!SILENT
 	if (debug_level >= 0) {
-		list_var(v, false);
+		list_var(v, 0);
 		printf(_("Solving equation #%d for (%s)...\n"), i + 1, var_str);
 	}
 #endif
 	want.level = 1;
 	want.kind = VARIABLE;
 	want.token.variable = v;
-	if (!solve_sub(&want, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i])) {
+	if (solve_sub(&want, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i]) <= 0) {
 		debug_string(0, _("Solve failed."));
 		return false;
 	}
@@ -1505,7 +1512,6 @@ long	v;
 	return true;
 }
 
-#if	!LIBRARY
 /*
  * The flist command.
  *
@@ -1527,16 +1533,21 @@ char	*cp;
 	for (; i <= j; i++) {
 		if (n_lhs[i] > 0) {
 			group_sub(i);
-			if (factor_flag) {
+			if (factor_flag || factor_int_flag) {
 				factor_int(lhs[i], &n_lhs[i]);
 				factor_int(rhs[i], &n_rhs[i]);
 			}
+#if	LIBRARY
+			if (gfp == stdout) {
+				result_str = list_equation(i, false);
+				return true;
+			}
+#endif
 			flist_sub(i);
 		}
 	}
 	return true;
 }
-#endif
 
 /*
  * The list command.
@@ -1546,9 +1557,8 @@ list_cmd(cp)
 char	*cp;
 {
 	int	i, j;
-	int	export_flag;
+	int	export_flag = false;
 
-	export_flag = false;
 	if (strcmp_tospace(cp, "export") == 0) {
 		export_flag = 2;
 		cp = skip_param(cp);
@@ -1563,7 +1573,7 @@ char	*cp;
 #if	LIBRARY
 		if (gfp == stdout) {
 			result_str = list_equation(i, export_flag);
-			continue;
+			return true;
 		}
 #endif
 		list1_sub(i, export_flag);
@@ -1571,7 +1581,7 @@ char	*cp;
 	return true;
 }
 
-#if	!BASICS
+#if	!BASICS && !LIBRARY
 /*
  * The code command.
  */
@@ -1580,17 +1590,18 @@ code_cmd(cp)
 char	*cp;
 {
 	int	i, j;
-	int	java_flag;
-	int	int_flag;
+	int	java_flag = 0;
+	int	int_flag = false;
 
-	java_flag = false;
-	int_flag = false;
 	if (strcmp_tospace(cp, "c") == 0) {
 		cp = skip_param(cp);
 	} else if (strcmp_tospace(cp, "java") == 0) {
 		cp = skip_param(cp);
-		java_flag = true;
-	} else if (strcmp_tospace(cp, "integer") == 0) {
+		java_flag = 1;
+	} else if (strcmp_tospace(cp, "python") == 0) {
+		cp = skip_param(cp);
+		java_flag = 2;
+	} else if (strcmp_tospace(cp, "int") == 0 || strcmp_tospace(cp, "integer") == 0) {
 		cp = skip_param(cp);
 		int_flag = true;
 	}
@@ -1600,7 +1611,9 @@ char	*cp;
 	for (; i <= j; i++) {
 		if (n_lhs[i] > 0) {
 			if (n_rhs[i] == 0 || n_lhs[i] != 1 || lhs[i][0].kind != VARIABLE) {
-				printf(_("Equation #%d is not a valid equation.\n"), i + 1);
+#if	!SILENT
+				printf(_("#%d is not a solved equation.\n"), i + 1);
+#endif
 				continue;
 			}
 			in_calc_cmd = true;
@@ -1612,7 +1625,9 @@ char	*cp;
 			in_calc_cmd = false;
 			if (int_flag) {
 				if (!int_expr(rhs[i], n_rhs[i])) {
-					printf(_("Equation #%d is not an integer equation.\n"), i + 1);
+#if	!SILENT
+					printf(_("#%d is not an integer equation.\n"), i + 1);
+#endif
 					continue;
 				}
 			}
@@ -1640,10 +1655,10 @@ char	*cp;
 	char	*cp1;
 
 	cp_start = cp;
-	i = cur_equation;
-	if (notdefined(i)) {
+	if (current_not_defined()) {
 		return false;
 	}
+	i = cur_equation;
 	if (strcmp_tospace(cp, "constants") == 0) {
 		cp = skip_param(cp);
 		if (extra_garbage(cp))
@@ -1658,6 +1673,7 @@ char	*cp;
 		if (strcmp_tospace(cp, "with") == 0) {
 			if (vc)
 				break;
+			error(_("No variables specified."));
 			return false;
 		}
 		if (vc >= ARR_CNT(va)) {
@@ -1724,7 +1740,7 @@ char	*cp;
 				goto do_this;
 			}
 		}
-		list_var(v, false);
+		list_var(v, 0);
 		snprintf(prompt_str, sizeof(prompt_str), _("Enter %s: "), var_str);
 		if (!get_expr(scratch, &n)) {
 			continue;
@@ -1761,18 +1777,9 @@ simplify_cmd(cp)
 char	*cp;
 {
 	int	i, j;
-	int	poly_flag;
-	int	quick_flag;
-	int	symb;
+	int	quick_flag = false, symb = false;
 
-	poly_flag = false;
-	quick_flag = false;
-	symb = false;
 	for (;; cp = skip_param(cp)) {
-		if (strncasecmp(cp, "poly", 4) == 0) {
-			poly_flag = true;
-			continue;
-		}
 		if (strncasecmp(cp, "symbolic", 4) == 0) {
 			symb = true;
 			continue;
@@ -1789,8 +1796,8 @@ char	*cp;
 	symb_flag = symb;
 	for (; i <= j; i++) {
 		if (n_lhs[i]) {
-			simpa_side(lhs[i], &n_lhs[i], poly_flag, quick_flag);
-			simpa_side(rhs[i], &n_rhs[i], poly_flag, quick_flag);
+			simpa_side(lhs[i], &n_lhs[i], quick_flag);
+			simpa_side(rhs[i], &n_rhs[i], quick_flag);
 			return_result(i);
 		}
 	}
@@ -1806,7 +1813,7 @@ factor_cmd(cp)
 char	*cp;
 {
 	int	i, j;
-	int	i1;
+	int	i1, count;
 	long	v;
 	double	d;
 	char	buf[MAX_CMD_LEN];
@@ -1815,29 +1822,18 @@ char	*cp;
 		cp = skip_param(cp);
 		if (*cp == '\0') {
 			my_strlcpy(prompt_str, _("Enter integers to factor: "), sizeof(prompt_str));
-			cp = getstring(buf, sizeof(buf));
+			cp = get_string(buf, sizeof(buf));
 			if (cp == NULL)
 				return false;
 		}
-		for (; *cp;) {
+		for (count = 1; *cp; count++) {
 			d = strtod(cp, &cp);
 			cp = skip_space(cp);
 			if (!factor_one(d)) {
 				error(_("Not a valid integer."));
 				return false;
 			}
-			fprintf(gfp, _("%.0f = "), d);
-			for (i = 0; i < uno;) {
-				fprintf(gfp, _("%.0f"), unique[i]);
-				if (ucnt[i] > 1) {
-					fprintf(gfp, "^%d", ucnt[i]);
-				}
-				i++;
-				if (i < uno) {
-					fprintf(gfp, " * ");
-				}
-			}
-			fprintf(gfp, "\n");
+			display_unique();
 		}
 		return true;
 	}
@@ -1989,7 +1985,7 @@ char	*cp;
 #endif
 		gfp = fp;
 		high_prec = true;
-		rv = list_cmd("all");
+		list_cmd("all");
 		high_prec = false;
 		gfp = stdout;
 		fclose(fp);
