@@ -1,5 +1,5 @@
 /*
- * Algebraic manipulator differentiation routines.
+ * Algebraic manipulator differentiation routines and commands.
  *
  * Copyright (c) 1987-2005 George Gesslein II.
  */
@@ -17,8 +17,8 @@ static int	d_recurse();
  */
 int
 differentiate(equation, np, v)
-token_type	*equation;	/* pointer to source and destination expression */
-int		*np;		/* pointer to the length of the expression */
+token_type	*equation;	/* pointer to source and destination equation side */
+int		*np;		/* pointer to the length of the equation side */
 long		v;		/* differentiation variable */
 {
 	int	i;
@@ -55,8 +55,7 @@ long		v;
 	int		i, j;
 	int		n;
 	int		op;
-	int		oploc;
-	int		endloc;
+	int		oploc, endloc;
 	complexs	c;
 
 	if (equation[loc].level < level) {
@@ -151,9 +150,7 @@ d_times:
 	equation[endloc].token.operatr = PLUS;
 	if (!d_recurse(equation, np, endloc + (oploc - loc) + 2, level + 2, v))
 		return false;
-	if (!d_recurse(equation, np, loc, level + 2, v))
-		return false;
-	return true;
+	return(d_recurse(equation, np, loc, level + 2, v));
 d_divide:
 /* Differentiate DIVIDE operator. */
 /* Use rule: d(u/v) = (v*d(u) - u*d(v))/v^2. */
@@ -186,9 +183,7 @@ d_divide:
 	equation[j].token.constant = 2.0;
 	if (!d_recurse(equation, np, endloc + (oploc - loc) + 2, level + 3, v))
 		return false;
-	if (!d_recurse(equation, np, loc, level + 3, v))
-		return false;
-	return true;
+	return(d_recurse(equation, np, loc, level + 3, v));
 d_power:
 /* Differentiate POWER operator. */
 /* Since we don't have logarithms, just do what we can without them. */
@@ -237,9 +232,7 @@ d_power:
 				for (i = loc; i < endloc; i++) {
 					equation[i].level++;
 				}
-				if (!d_recurse(equation, np, n, level + 1, v))
-					return false;
-				return true;
+				return(d_recurse(equation, np, n, level + 1, v));
 			}
 			return false;
 		}
@@ -285,9 +278,7 @@ d_power:
 	blt(&equation[loc+n], &equation[endloc], (*np - endloc) * sizeof(token_type));
 	*np += loc + n - endloc;
 	blt(&equation[loc], scratch, n * sizeof(token_type));
-	if (!d_recurse(equation, np, loc + j, level + 1, v))
-		return false;
-	return true;
+	return(d_recurse(equation, np, loc + j, level + 1, v));
 }
 
 /*
@@ -297,15 +288,13 @@ int
 derivative_cmd(cp)
 char	*cp;
 {
-	long		v;
-	int		i;
-	long		l1, order;
+	long		v = 0;
+	int		i, j;
+	long		l1, order = 1;
 	token_type	*source, *dest;
 	int		n1, *nps, *np;
 
-	v = 0;
-	order = 1;
-	if (notdefined(cur_equation)) {
+	if (current_not_defined()) {
 		return false;
 	}
 	i = next_espace();
@@ -322,12 +311,14 @@ char	*cp;
 	}
 	if (*cp) {
 		if (is_all(cp)) {
-			v = MATCH_ANY;
 			cp = skip_param(cp);
+			v = MATCH_ANY;
 		} else {
-			cp = parse_var2(&v, cp);
-			if (cp == NULL) {
-				return false;
+			if (!(isascii(*cp) && isdigit(*cp))) {
+				cp = parse_var2(&v, cp);
+				if (cp == NULL) {
+					return false;
+				}
 			}
 		}
 		if (*cp) {
@@ -338,6 +329,22 @@ char	*cp;
 			return false;
 		}
 	}
+	if (v == 0) {
+		for (j = 0; j < *nps; j += 2) {
+			if (source[j].kind == VARIABLE) {
+				if ((source[j].token.variable & VAR_MASK) <= SIGN)
+					continue;
+				if (v) {
+					if (v != source[j].token.variable) {
+						v = 0;
+						break;
+					}
+				} else {
+					v = source[j].token.variable;
+				}
+			}
+		}
+	}
 	if (v == 0) {
 		if (!prompt_var(&v)) {
 			return false;
@@ -347,9 +354,15 @@ char	*cp;
 		error(_("Variable not found.  Result would be zero."));
 		return false;
 	}
+#if	!SILENT
+	list_var(v, 0);
+	printf(_("Differentiating with respect to (%s) and simplifying"), var_str);
 	if (n_rhs[cur_equation]) {
-		debug_string(0, _("Differentiating and simplifying the RHS..."));
+		printf(_(" the RHS...\n"));
+	} else {
+		printf("...\n");
 	}
+#endif
 	blt(dest, source, *nps * sizeof(token_type));
 	n1 = *nps;
 	for (l1 = 0; l1 < order; l1++) {
@@ -357,7 +370,7 @@ char	*cp;
 			error(_("Differentiation failed."));
 			return false;
 		}
-		simpa_side(dest, &n1, false, true);
+		simpa_side(dest, &n1, true);
 	}
 	if (n_rhs[cur_equation]) {
 		blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
@@ -376,18 +389,18 @@ int
 extrema_cmd(cp)
 char	*cp;
 {
-	int		i;
-	long		v;
+	int		i, j;
+	long		v = 0;
 	token_type	want;
 	token_type	*source;
 	int		n;
 
-	if (notdefined(cur_equation)) {
+	if (current_not_defined()) {
 		return false;
 	}
 	if (n_rhs[cur_equation]) {
 		if (!solved_equation(cur_equation)) {
-			error(_("Current equation is not solved for a variable."));
+			error(_("The current equation is not solved for a variable."));
 			return false;
 		}
 		source = rhs[cur_equation];
@@ -401,13 +414,30 @@ char	*cp;
 		if (cp == NULL) {
 			return false;
 		}
-	} else {
-		if (!prompt_var(&v)) {
+		if (extra_garbage(cp)) {
 			return false;
 		}
 	}
-	if (extra_garbage(cp)) {
-		return false;
+	if (v == 0) {
+		for (j = 0; j < n; j += 2) {
+			if (source[j].kind == VARIABLE) {
+				if ((source[j].token.variable & VAR_MASK) <= SIGN)
+					continue;
+				if (v) {
+					if (v != source[j].token.variable) {
+						v = 0;
+						break;
+					}
+				} else {
+					v = source[j].token.variable;
+				}
+			}
+		}
+	}
+	if (v == 0) {
+		if (!prompt_var(&v)) {
+			return false;
+		}
 	}
 	if (!found_var(source, n, v)) {
 		error(_("Variable not found."));
@@ -420,29 +450,26 @@ char	*cp;
 		error(_("Differentiation failed."));
 		return false;
 	}
-	simpa_side(rhs[i], &n_rhs[i], false, true);
+	simpa_side(rhs[i], &n_rhs[i], true);
 	if (!found_var(rhs[i], n_rhs[i], v)) {
 		error(_("There are no extrema for this equation."));
 		return false;
 	}
 	n_lhs[i] = 1;
-	lhs[i][0].level = 1;
-	lhs[i][0].kind = CONSTANT;
-	lhs[i][0].token.constant = 0.0;
+	lhs[i][0] = zero_token;
 	cur_equation = i;
 	want.level = 1;
 	want.kind = VARIABLE;
 	want.token.variable = v;
-	if (!solve_sub(&want, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i])) {
+	if (solve_sub(&want, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i]) <= 0) {
 		error(_("Solve failed."));
 		return false;
 	}
-	simpa_side(rhs[i], &n_rhs[i], false, false);
+	simpa_side(rhs[i], &n_rhs[i], false);
 	return_result(cur_equation);
 	return true;
 }
 
-#if	!BASICS
 /*
  * The taylor command.
  */
@@ -450,29 +477,31 @@ int
 taylor_cmd(cp)
 char	*cp;
 {
-	long		v;
-	int		i, j, k;
-	int		i1, j1;
+	long		v = 0;
+	int		i, j, k, i1;
 	int		level;
-	int		n;
-	int		order;
-	token_type	*ep;
+	long		l1, n, order = -1L;
 	double		d;
-	char		*cp1, buf[MAX_CMD_LEN];
+	char		*cp_start, *cp1, buf[MAX_CMD_LEN];
 	int		our;
 	int		our_nlhs, our_nrhs;
-	token_type	*source, *dest;
+	token_type	*ep, *source, *dest;
 	int		n1, *nps, *np;
 
-	if (notdefined(cur_equation)) {
+	cp_start = cp;
+	if (current_not_defined()) {
 		return false;
 	}
 	i = next_espace();
 	blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
 	n_lhs[i] = n_lhs[cur_equation];
 	n_rhs[i] = 0;
-	our = next_espace();
+        our = alloc_next_espace();
 	n_lhs[i] = 0;
+        if (our < 0) {
+                error(_("Out of free equation spaces."));
+		return false;
+	}
 	if (n_rhs[cur_equation]) {
 		source = rhs[cur_equation];
 		nps = &n_rhs[cur_equation];
@@ -484,15 +513,38 @@ char	*cp;
 		dest = lhs[i];
 		np = &n_lhs[i];
 	}
-	if (*cp == '\0') {
-		if (!prompt_var(&v))
-			return false;
-	} else {
+	if (*cp && !(isascii(*cp) && isdigit(*cp))) {
 		cp = parse_var2(&v, cp);
 		if (cp == NULL) {
 			return false;
 		}
-		if (extra_garbage(cp))
+	}
+	if (*cp) {
+		order = decstrtol(cp, &cp1);
+		if (cp1 != skip_param(cp) || order < 0) {
+			error(_("Positive integer required for order."));
+			return false;
+		}
+		cp = cp1;
+	}
+	if (v == 0) {
+		for (j = 0; j < *nps; j += 2) {
+			if (source[j].kind == VARIABLE) {
+				if ((source[j].token.variable & VAR_MASK) <= SIGN)
+					continue;
+				if (v) {
+					if (v != source[j].token.variable) {
+						v = 0;
+						break;
+					}
+				} else {
+					v = source[j].token.variable;
+				}
+			}
+		}
+	}
+	if (v == 0) {
+		if (!prompt_var(&v))
 			return false;
 	}
 	if (!found_var(source, *nps, v)) {
@@ -505,26 +557,41 @@ char	*cp;
 		error(_("Differentiation failed."));
 		return false;
 	}
-	list_var(v, false);
-	printf(_("Taylor approximation of current equation about %s = point.\n"), var_str);
-oops:
-	my_strlcpy(prompt_str, _("Enter point: "), sizeof(prompt_str));
-	if (!get_expr(lhs[our], &our_nlhs)) {
-		printf(_("Enter 0 if you don't know.\n"));
-		goto oops;
-	}
-	my_strlcpy(prompt_str, _("Enter order (number of derivatives to take): "), sizeof(prompt_str));
-	if ((cp1 = getstring(buf, sizeof(buf))) == NULL)
-		return false;
-	if (*cp1 == '\0') {
-		printf(_("Derivatives will be taken until they reach zero.\n"));
-		order = INT_MAX;
-	} else {
-		order = decstrtol(cp1, &cp);
-		if (cp == cp1 || order < 0) {
-			error(_("Positive integer required."));
+	if (*cp) {
+		input_column += (cp - cp_start);
+		if (!case_sensitive_flag) {
+			str_tolower(cp);
+		}
+		if ((cp = parse_section(lhs[our], &our_nlhs, cp)) == NULL || our_nlhs <= 0) {
 			return false;
 		}
+		if (extra_garbage(cp))
+			return false;
+	} else {
+#if	!SILENT
+		list_var(v, 0);
+		printf(_("Taylor series expansion about (%s) = point.\n"), var_str);
+#endif
+		my_strlcpy(prompt_str, _("Enter point: "), sizeof(prompt_str));
+		if (!get_expr(lhs[our], &our_nlhs)) {
+			error(_("Enter 0 if you don't know."));
+			return false;
+		}
+	}
+	if (order < 0) {
+		my_strlcpy(prompt_str, _("Enter order (number of derivatives to take): "), sizeof(prompt_str));
+		if ((cp1 = get_string(buf, sizeof(buf))) == NULL)
+			return false;
+		if (*cp1) {
+			order = decstrtol(cp1, &cp);
+			if (*cp || order < 0) {
+				error(_("Positive integer required for order."));
+				return false;
+			}
+		} else {
+			printf(_("Derivatives will be taken until they reach zero...\n"));
+			order = LONG_MAX - 1L;
+		}
 	}
 	n = 0;
 	i1 = 0;
@@ -538,11 +605,11 @@ loop_again:
 				error_huge();
 			blt(&dest[k+our_nlhs], &dest[k+1], (n1 - (k + 1)) * sizeof(token_type));
 			n1 += our_nlhs - 1;
-			j1 = k;
+			j = k;
 			blt(&dest[k], lhs[our], our_nlhs * sizeof(token_type));
 			k += our_nlhs;
-			for (; j1 < k; j1++)
-				dest[j1].level += level;
+			for (; j < k; j++)
+				dest[j].level += level;
 			k--;
 		}
 	}
@@ -550,8 +617,7 @@ loop_again:
 		error_huge();
 	for (k = i1; k < n1; k++)
 		dest[k].level++;
-	k = n1;
-	ep = &dest[k];
+	ep = &dest[n1];
 	ep->level = 1;
 	ep->kind = OPERATOR;
 	ep->token.operatr = TIMES;
@@ -563,13 +629,13 @@ loop_again:
 	ep->level = 3;
 	ep->kind = OPERATOR;
 	ep->token.operatr = MINUS;
-	k += 3;
-	j1 = k;
-	blt(&dest[k], lhs[our], our_nlhs * sizeof(token_type));
-	k += our_nlhs;
-	for (; j1 < k; j1++)
-		dest[j1].level += 3;
-	ep = &dest[k];
+	n1 += 3;
+	j = n1;
+	blt(&dest[n1], lhs[our], our_nlhs * sizeof(token_type));
+	n1 += our_nlhs;
+	for (; j < n1; j++)
+		dest[j].level += 3;
+	ep = &dest[n1];
 	ep->level = 2;
 	ep->kind = OPERATOR;
 	ep->token.operatr = POWER;
@@ -582,14 +648,13 @@ loop_again:
 	ep->kind = OPERATOR;
 	ep->token.operatr = DIVIDE;
 	ep++;
-	for (d = 1.0, j = 2; j <= n; j++)
-		d *= j;
+	for (d = 1.0, l1 = 2; l1 <= n; l1++)
+		d *= l1;
 	ep->level = 1;
 	ep->kind = CONSTANT;
 	ep->token.constant = d;
-	k += 4;
-	n1 = k;
-	for (; i1 < k; i1++)
+	n1 += 4;
+	for (; i1 < n1; i1++)
 		dest[i1].level++;
 	simp_loop(dest, &n1);
 	side_debug(1, dest, n1);
@@ -600,21 +665,19 @@ loop_again:
 				return false;
 			}
 		}
-		simpa_side(rhs[our], &our_nrhs, false, true);
-		if (our_nrhs != 1 || rhs[our][0].kind != CONSTANT
-		    || rhs[our][0].token.constant != 0.0) {
-			k = n1;
-			if ((k + 1 + our_nrhs) > n_tokens)
+		simpa_side(rhs[our], &our_nrhs, true);
+		if (our_nrhs != 1 || rhs[our][0].kind != CONSTANT || rhs[our][0].token.constant != 0.0) {
+			i1 = n1;
+			if ((i1 + 1 + our_nrhs) > n_tokens)
 				error_huge();
-			for (i1 = 0; i1 < k; i1++)
-				dest[i1].level++;
-			dest[k].level = 1;
-			dest[k].kind = OPERATOR;
-			dest[k].token.operatr = PLUS;
-			k++;
-			i1 = k;
-			blt(&dest[k], rhs[our], our_nrhs * sizeof(token_type));
-			n1 = k + our_nrhs;
+			for (j = 0; j < i1; j++)
+				dest[j].level++;
+			dest[i1].level = 1;
+			dest[i1].kind = OPERATOR;
+			dest[i1].token.operatr = PLUS;
+			i1++;
+			blt(&dest[i1], rhs[our], our_nrhs * sizeof(token_type));
+			n1 = i1 + our_nrhs;
 			n++;
 			goto loop_again;
 		}
@@ -627,7 +690,6 @@ loop_again:
 	return_result(cur_equation);
 	return true;
 }
-#endif
 
 /*
  * The limit command.
@@ -641,13 +703,11 @@ char	*cp;
 	token_type	solved_v, want;
 	char		*cp_start;
 	int		nl;
-	int		infinity_flag;
-	int		found_den, complex_den;
-	int		num, den;
+	int		infinity_flag, found_den, complex_den, num, den;
 	int		level;
 
 	cp_start = cp;
-	if (notdefined(cur_equation)) {
+	if (current_not_defined()) {
 		return false;
 	}
 	if (n_rhs[cur_equation] == 0) {
@@ -659,10 +719,10 @@ char	*cp;
 		lhs[cur_equation][0].token.variable = V_ANSWER;
 	}
 	if (!solved_equation(cur_equation)) {
-		error(_("Current equation is not solved for a variable."));
+		error(_("The current equation is not solved for a variable."));
 		return false;
 	}
-	blt(&solved_v, lhs[cur_equation], sizeof(token_type));
+	solved_v = lhs[cur_equation][0];
 	i = next_espace();
 	if (*cp) {
 		cp = parse_var2(&v, cp);
@@ -695,14 +755,8 @@ char	*cp;
 	if (extra_garbage(cp))
 		return false;
 	simp_loop(lhs[i], &nl);
-	infinity_flag = false;
-	for (j = 0; j < nl; j += 2) {
-		if (lhs[i][j].kind == CONSTANT
-		    && isinf(lhs[i][j].token.constant)) {
-			infinity_flag = true;
-		}
-	}
-	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
+	infinity_flag = exp_contains_infinity(lhs[i], nl);
+	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
 	group_proc(rhs[cur_equation], &n_rhs[cur_equation]);
 	found_den = false;
 	complex_den = false;
@@ -743,18 +797,14 @@ char	*cp;
 	}
 	if (!found_den || (infinity_flag && !complex_den)) {
 		subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], lhs[i], nl, v);
-		simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
+		simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
 		return_result(cur_equation);
 		return true;
 	}
 	if (infinity_flag) {
 		nl = 1;
-		lhs[i][0].level = 1;
-		lhs[i][0].kind = CONSTANT;
-		lhs[i][0].token.constant = 0.0;
-		tlhs[0].level = 1;
-		tlhs[0].kind = CONSTANT;
-		tlhs[0].token.constant = 1.0;
+		lhs[i][0] = zero_token;
+		tlhs[0] = one_token;
 		tlhs[1].level = 1;
 		tlhs[1].kind = OPERATOR;
 		tlhs[1].token.operatr = DIVIDE;
@@ -767,21 +817,21 @@ char	*cp;
 	want.level = 1;
 	want.kind = VARIABLE;
 	want.token.variable = v;
-	if (!solve_sub(&want, 1, lhs[cur_equation], &n_lhs[cur_equation], rhs[cur_equation], &n_rhs[cur_equation])) {
+	if (solve_sub(&want, 1, lhs[cur_equation], &n_lhs[cur_equation], rhs[cur_equation], &n_rhs[cur_equation]) <= 0) {
 		error(_("Can't take the limit because solve failed."));
 		return false;
 	}
 	blt(lhs[cur_equation], lhs[i], nl * sizeof(token_type));
 	n_lhs[cur_equation] = nl;
 	symb_flag = true;
-	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
+	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
 	symb_flag = false;
-	if (!solve_sub(&solved_v, 1, lhs[cur_equation], &n_lhs[cur_equation], rhs[cur_equation], &n_rhs[cur_equation])) {
+	if (solve_sub(&solved_v, 1, lhs[cur_equation], &n_lhs[cur_equation], rhs[cur_equation], &n_rhs[cur_equation]) <= 0) {
 		error(_("Can't take the limit because solve failed."));
 		return false;
 	}
 	symb_flag = true;
-	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
+	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
 	symb_flag = false;
 	return_result(cur_equation);
 	return true;
