@@ -22,9 +22,13 @@ extern token_type	*trhs;
 
 extern token_type	*scratch;
 
+extern token_type	zero_token;
+extern token_type	one_token;
+
 extern char		*var_names[MAX_VAR_NAMES];
 
 extern int		case_sensitive_flag;
+extern int		factor_int_flag;
 extern int		groupall;
 extern int		in_calc_cmd;
 extern int		preserve_roots;
@@ -32,6 +36,7 @@ extern int		true_modulus;
 extern int		screen_columns;
 extern int		screen_rows;
 extern int		finance_option;
+extern int		starstar;
 extern int		debug_level;
 extern int		domain_check;
 extern int		color_flag;
@@ -46,7 +51,7 @@ extern int		sign_flag;
 extern double		small_epsilon;
 extern double		epsilon;
 extern char		var_str[MAX_VAR_LEN+80];
-extern char		prompt_str[MAX_VAR_LEN+80];
+extern char		prompt_str[MAX_PROMPT_LEN];
 extern char		*dir_path;
 extern char		*prog_name;
 
