@@ -1,5 +1,5 @@
 /*
- * Algebraic manipulator factorizing routines.
+ * Algebraic manipulator symbolic factorizing routines.
  *
  * Copyright (c) 1987-2005 George Gesslein II.
  *
@@ -9,7 +9,6 @@
 
 #include "includes.h"
 
-static void	try_factor();
 static int	fplus_recurse();
 static int	fplus_sub();
 static int	big_fplus();
@@ -17,172 +16,6 @@ static int	ftimes_recurse();
 static int	ftimes_sub();
 static int	fpower_recurse();
 static int	fpower_sub();
-static int	fc_recurse();
-
-static double nn, vv;
-static double sq[] = {
-	10, 2, 4, 2, 4, 6, 2, 6,
-	 4, 2, 4, 6, 6, 2, 6, 4,
-	 2, 6, 4, 6, 8, 4, 2, 4,
-	 2, 4, 8, 6, 4, 6, 2, 4,
-	 6, 2, 6, 6, 4, 2, 4, 6,
-	 2, 6, 4, 2, 4, 2,10, 2
-};
-
-/*
- * Factor the integer in "start".
- * Store unique factors in the "unique" array.
- * Return true if successful.
- */
-int
-factor_one(start)
-double	start;
-{
-	int	i;
-	double	ii;
-
-	uno = 0;
-	nn = start;
-	if (nn == 0.0)
-		return false;
-	if (fabs(nn) >= MAX_K_INTEGER) {
-		/* too large to factor */
-		return false;
-	}
-	if (fmod(nn, 1.0) != 0.0) {
-		/* not an integer */
-		return false;
-	}
-	vv = 1.0 + sqrt(fabs(nn));
-	try_factor(2.0);
-	try_factor(3.0);
-	try_factor(5.0);
-	try_factor(7.0);
-	ii = 1.0;
-	while (ii <= vv) {
-		for (i = 0; i < 48; i++) {
-			ii += sq[i];
-			try_factor(ii);
-		}
-	}
-	if (nn != 1.0) {
-		try_factor(nn);
-	}
-	return true;
-}
-
-/*
- * See if "arg" is one or more factors of "nn".
- * If so, save it and remove it from "nn".
- */
-static void
-try_factor(arg)
-double	arg;
-{
-	while (fmod(nn, arg) == 0.0) {
-		if (uno > 0 && unique[uno-1] == arg) {
-			ucnt[uno-1]++;
-		} else {
-			unique[uno] = arg;
-			ucnt[uno] = 1;
-			uno++;
-		}
-		nn /= arg;
-		vv = 1.0 + sqrt(fabs(nn));
-		if (nn <= 1.0 && nn >= -1.0)
-			break;
-	}
-}
-
-/*
- * Convert unique[] back into an integer.
- */
-double
-multiply_out_unique()
-{
-	int	i, j;
-	double	d;
-
-	d = 1.0;
-	for (i = 0; i < uno; i++) {
-		for (j = 0; j < ucnt[i]; j++) {
-			d *= unique[i];
-		}
-	}
-	return d;
-}
-
-/*
- * Factor integers in an expression.
- *
- * Return true if expression was modified.
- */
-int
-factor_int(equation, np)
-token_type	*equation;
-int		*np;
-{
-	int	i, j;
-	int	xsize;
-	int	level;
-	int	modified;
-
-	modified = false;
-	for (i = 0; i < *np; i += 2) {
-		if (equation[i].kind == CONSTANT && factor_one(equation[i].token.constant) && uno > 0) {
-			if (uno == 1 && ucnt[0] <= 1)
-				continue;	/* prime number */
-			level = equation[i].level;
-			if (uno > 1 && *np > 1)
-				level++;
-			xsize = -2;
-			for (j = 0; j < uno; j++) {
-				if (ucnt[j] > 1)
-					xsize += 4;
-				else
-					xsize += 2;
-			}
-			if (*np + xsize > n_tokens) {
-				error_huge();
-			}
-			for (j = 0; j < uno; j++) {
-				if (ucnt[j] > 1)
-					xsize = 4;
-				else
-					xsize = 2;
-				if (j == 0)
-					xsize -= 2;
-				if (xsize > 0) {
-					blt(&equation[i+xsize], &equation[i], (*np - i) * sizeof(token_type));
-					*np += xsize;
-					if (j > 0) {
-						i++;
-						equation[i].kind = OPERATOR;
-						equation[i].level = level;
-						equation[i].token.operatr = TIMES;
-						i++;
-					}
-				}
-				equation[i].kind = CONSTANT;
-				equation[i].level = level;
-				equation[i].token.constant = unique[j];
-				if (ucnt[j] > 1) {
-					equation[i].level = level + 1;
-					i++;
-					equation[i].kind = OPERATOR;
-					equation[i].level = level + 1;
-					equation[i].token.operatr = POWER;
-					i++;
-					equation[i].level = level + 1;
-					equation[i].kind = CONSTANT;
-					equation[i].token.constant = ucnt[j];
-				}
-			}
-			modified = true;
-		}
-	}
-	return modified;
-}
 
 /*
  * Factor divides only.
@@ -240,6 +73,12 @@ double		d;
 	return fplus_recurse(equation, np, 0, 1, v, d, 0);
 }
 
+/*
+ * Recursively factor at "level" of parentheses and deeper,
+ * beginning at index "loc".
+ *
+ * Return true if the expression was modified.
+ */
 static int
 fplus_recurse(equation, np, loc, level, v, d, factor_code)
 token_type	*equation;
@@ -248,12 +87,11 @@ long		v;
 double		d;
 int		factor_code;
 {
-	int	modified;
+	int	modified = false;
 	int	i, j, k;
 	int	op;
 	int	len1, len2;
 
-	modified = false;
 	op = 0;
 	for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
 		if (equation[i].level == level) {
@@ -304,11 +142,20 @@ f_again:
 	return modified;
 }
 
+/*
+ * Do the factoring of two subexpressions added together.
+ *
+ * Return true if a transformation was made.
+ */
 static int
 fplus_sub(equation, np, loc, i1, n1, i2, n2, level, whole_flag, div_only, v, d)
-token_type	*equation;
-int		*np, loc, i1, n1, i2, n2;
-int		level, whole_flag, div_only;
+token_type	*equation;	/* the entire expression */
+int		*np;		/* pointer to length of the entire expression */
+int		loc;		/* index into the beginning of this additive subexpression */
+int		i1, n1, i2, n2;
+int		level;
+int		whole_flag;	/* factor only whole expressions multiplied by a constant */
+int		div_only;	/* factor only divides */
 long		v;
 double		d;
 {
@@ -330,13 +177,6 @@ double		d;
 	e1 = i1 + n1;
 	e2 = i2 + n2;
 	op2 = equation[i2-1].token.operatr;
-	switch (op2) {
-	case PLUS:
-	case MINUS:
-		break;
-	default:
-		return false;
-	}
 	if (i1 <= loc) {
 		op1 = PLUS;
 	} else
@@ -368,10 +208,8 @@ f_outer:
 		sop1 = TIMES;
 	else
 		sop1 = equation[b1-1].token.operatr;
-	if (div_only && sop1 != DIVIDE)
-		goto f_outer;
-	if (i - b1 == 1 && equation[b1].kind == CONSTANT
-	    && fabs(equation[b1].token.constant) == 1.0) {
+	if ((div_only && sop1 != DIVIDE)
+	    || (i - b1 == 1 && equation[b1].kind == CONSTANT && fabs(equation[b1].token.constant) == 1.0)) {
 		goto f_outer;
 	}
 	if (!whole_flag && v && (v != MATCH_ANY)) {
@@ -379,8 +217,7 @@ f_outer:
 			for (k = b1;; k += 2) {
 				if (k >= i)
 					goto f_outer;
-				if (equation[k].kind == VARIABLE
-				    && equation[k].token.variable == v) {
+				if (equation[k].kind == VARIABLE && equation[k].token.variable == v) {
 					break;
 				}
 			}
@@ -434,8 +271,7 @@ f_inner:
 				goto f_inner;
 		}
 	}
-	if (j - b2 == 1 && equation[b2].kind == CONSTANT
-	    && fabs(equation[b2].token.constant) == 1.0) {
+	if (j - b2 == 1 && equation[b2].kind == CONSTANT && fabs(equation[b2].token.constant) == 1.0) {
 		goto f_inner;
 	}
 	ai = i;
@@ -464,6 +300,7 @@ f_inner:
 		}
 	}
 	if (same_flag) {
+		/* do the factor transformation */
 		power = 1.0;
 more_power:
 		if (sop1 == DIVIDE) {
@@ -508,9 +345,10 @@ more_power:
 			m = len + ai - b1;
 			blt(&scratch[len], &equation[b1], (i - b1) * sizeof(*equation));
 			len += i - b1;
-			if (b1 == i1 && i == e1)
+			if (b1 == i1 && i == e1) {
 				for (; l < len; l++)
 					scratch[l].level++;
+			}
 			l = m;
 			m++;
 			for (; m < len; m++)
@@ -706,6 +544,10 @@ end_mess:
 	goto f_inner;
 }
 
+/*
+ * Factor transformation for a more general pair of subexpressions added together
+ * with a common base.
+ */
 static int
 big_fplus(equation, level, diff_sign, sop1, op1, op2, i1, i2, b1, b2, ai, aj, i, j, e1, e2)
 token_type	*equation;
@@ -751,8 +593,9 @@ int		e1, e2;
 	scratch[len].kind = CONSTANT;
 	if (op1 == MINUS) {
 		scratch[len].token.constant = -1.0;
-	} else
+	} else {
 		scratch[len].token.constant = 1.0;
+	}
 	len++;
 	blt(&scratch[len], &equation[i], (e1 - i) * sizeof(*equation));
 	len += e1 - i;
@@ -831,12 +674,11 @@ ftimes_recurse(equation, np, loc, level)
 token_type	*equation;
 int		*np, loc, level;
 {
-	int	modified;
+	int	modified = false;
 	int	i, j, k;
 	int	op;
 	int	len1, len2;
 
-	modified = false;
 	op = 0;
 	for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
 		if (equation[i].level == level) {
@@ -902,20 +744,12 @@ int		*np, loc, i1, n1, i2, n2, level;
 	e1 = i1 + n1;
 	e2 = i2 + n2;
 	op2 = equation[i2-1].token.operatr;
-	switch (op2) {
-	case TIMES:
-	case DIVIDE:
-		break;
-	default:
-		return false;
-	}
 	if (i1 <= loc) {
 		op1 = TIMES;
 	} else {
 		op1 = equation[i1-1].token.operatr;
 	}
-	if ((n1 == 1 && equation[i1].kind == CONSTANT)
-	    || (n2 == 1 && equation[i2].kind == CONSTANT)) {
+	if ((n1 == 1 && equation[i1].kind == CONSTANT) || (n2 == 1 && equation[i2].kind == CONSTANT)) {
 		return false;
 	}
 	both_divide = (op1 == DIVIDE && op2 == DIVIDE);
@@ -925,14 +759,12 @@ int		*np, loc, i1, n1, i2, n2, level;
 		goto common_base;
 	}
 	for (i = i1 + 1; i < e1; i += 2) {
-		if (equation[i].level == level
-		    && equation[i].token.operatr == POWER) {
+		if (equation[i].level == level && equation[i].token.operatr == POWER) {
 			break;
 		}
 	}
 	for (j = i2 + 1; j < e2; j += 2) {
-		if (equation[j].level == level
-		    && equation[j].token.operatr == POWER) {
+		if (equation[j].level == level && equation[j].token.operatr == POWER) {
 			break;
 		}
 	}
@@ -942,22 +774,11 @@ int		*np, loc, i1, n1, i2, n2, level;
 	if (se_compare(&equation[i1], i - i1, &equation[i2], j - i2, &diff_sign)) {
 		goto common_base;
 	}
-#if	false
-	if (i != e1 && se_compare(&equation[i1], e1 - i1, &equation[i2], j - i2, &diff_sign)) {
-		i = e1;
-		goto common_base;
-	}
-	if (j != e2 && se_compare(&equation[i1], i - i1, &equation[i2], e2 - i2, &diff_sign)) {
-		j = e2;
-		goto common_base;
-	}
-#endif
 	return false;
 
 common_base:
 	rlen1 = ((i == e1) ? 1 : (e1 - i - 1));
-	len = (i - i1) + 1 + ((op1 == DIVIDE && !both_divide) ? 2 : 0)
-	    + rlen1 + 1
+	len = (i - i1) + 1 + ((op1 == DIVIDE && !both_divide) ? 2 : 0) + rlen1 + 1
 	    + ((j == e2) ? 1 : (e2 - j - 1));
 	len -= n1;
 	if (j - i2 == 1 && equation[i2].kind == CONSTANT && equation[i2].token.constant == -1.0)
@@ -1045,12 +866,11 @@ fpower_recurse(equation, np, loc, level)
 token_type	*equation;
 int		*np, loc, level;
 {
-	int	modified;
+	int	modified = false;
 	int	i, j, k;
 	int	op;
 	int	len1, len2;
 
-	modified = false;
 	op = 0;
 	for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
 		if (equation[i].level == level) {
@@ -1113,38 +933,28 @@ int		*np, loc, i1, n1, i2, n2, level;
 	int		len;
 	int		diff_sign;
 	int		all_divide;
-	token_type	temp;
 
 	e1 = i1 + n1;
 	e2 = i2 + n2;
 	op2 = equation[i2-1].token.operatr;
-	switch (op2) {
-	case TIMES:
-	case DIVIDE:
-		break;
-	default:
-		return false;
-	}
 	if (i1 <= loc) {
 		op1 = TIMES;
 	} else
 		op1 = equation[i1-1].token.operatr;
-	for (i = i1; i < e1; i++) {
-		if (equation[i].level == level
-		    && equation[i].kind == OPERATOR
-		    && equation[i].token.operatr == POWER) {
+	for (i = i1 + 1;; i += 2) {
+		if (i >= e1)
+			return false;
+		if (equation[i].level == level && equation[i].token.operatr == POWER) {
 			break;
 		}
 	}
-	for (j = i2; j < e2; j++) {
-		if (equation[j].level == level
-		    && equation[j].kind == OPERATOR
-		    && equation[j].token.operatr == POWER) {
+	for (j = i2 + 1;; j += 2) {
+		if (j >= e2)
+			return false;
+		if (equation[j].level == level && equation[j].token.operatr == POWER) {
 			break;
 		}
 	}
-	if (i >= e1 || j >= e2)
-		return false;
 	start2 = j;
 fp_outer:
 	pop1 = equation[i].token.operatr;
@@ -1165,10 +975,7 @@ fp_outer:
 		}
 	}
 #endif
-	temp.level = 1;
-	temp.kind = CONSTANT;
-	temp.token.constant = 1.0;
-	if (se_compare(&equation[b1], i - b1, &temp, 1, &diff_sign)) {
+	if (se_compare(&equation[b1], i - b1, &one_token, 1, &diff_sign)) {
 		goto fp_outer;
 	}
 	j = start2;
@@ -1266,218 +1073,3 @@ fp_inner:
 	}
 	goto fp_inner;
 }
-
-/*
- * Factor constants in equation side.
- *
- * This routine is often necessary because the expression compare (se_compare())
- * does not return a multiplier (except for +/-1.0).
- * This routine is not used during polynomial operations.
- * It is required for simplification of algebraic fractions, etc.
- *
- * If "level_code" is 0, all additive expressions are normalized
- * by making at least one coefficient unity by factoring out
- * the smallest constant.
- *
- * If "level_code" is 1, any level 1 additive expression is factored
- * nicely for readability, while all deeper levels are normalized.
- *
- * If "level_code" is 2, nothing is normalized unless it increases
- * readability.
- *
- * If "level_code" is 3, nothing is done.
- *
- * Return true if equation side was modified.
- */
-int
-factor_constants(equation, np, level_code)
-token_type	*equation;
-int		*np;
-int		level_code;
-{
-	if (level_code > 2)
-		return false;
-	return fc_recurse(equation, np, 0, 1, level_code);
-}
-
-static int
-fc_recurse(equation, np, loc, level, level_code)
-token_type	*equation;
-int		*np, loc, level;
-int		level_code;
-{
-	int	modified;
-	int	i, j, k;
-	int	op;
-	int	neg_flag;
-	double	minimum;
-	double	d;
-	int	first;
-	int	count;
-
-	modified = false;
-	for (i = loc; i < *np && equation[i].level >= level;) {
-		if (equation[i].level > level) {
-			modified |= fc_recurse(equation, np, i, level + 1, level_code);
-			i++;
-			for (; i < *np && equation[i].level > level; i += 2)
-				;
-			continue;
-		}
-		i++;
-	}
-	if (modified)
-		return true;
-	minimum = 1.0;
-	neg_flag = true;
-	first = true;
-	count = 0;
-	for (i = loc;;) {
-break_cont:
-		if (i >= *np || equation[i].level < level)
-			break;
-		if (equation[i].level == level) {
-			switch (equation[i].kind) {
-			case CONSTANT:
-				if (i == loc && equation[i].token.constant >= 0.0)
-					neg_flag = false;
-				d = fabs(equation[i].token.constant);
-				if (first) {
-					minimum = d;
-					first = false;
-				} else if (d < minimum) {
-					minimum = d;
-				}
-				break;
-			case OPERATOR:
-				count++;
-				switch (equation[i].token.operatr) {
-				case PLUS:
-					neg_flag = false;
-				case MINUS:
-					break;
-				default:
-					return modified;
-				}
-				break;
-			default:
-				if (i == loc)
-					neg_flag = false;
-				if (first) {
-					minimum = 1.0;
-					first = false;
-				} else if (1.0 < minimum)
-					minimum = 1.0;
-				break;
-			}
-		} else {
-			op = 0;
-			for (j = i + 1;; j += 2) {
-				if (j >= *np || equation[j].level <= level)
-					break;
-				if (equation[j].level == level + 1) {
-					op = equation[j].token.operatr;
-				}
-			}
-			if (op == TIMES || op == DIVIDE) {
-				for (k = i; k < j; k++) {
-					if (equation[k].level == (level + 1)
-					    && equation[k].kind == CONSTANT) {
-						if (i == loc && equation[k].token.constant >= 0.0)
-							neg_flag = false;
-						d = fabs(equation[k].token.constant);
-						if (first) {
-							minimum = d;
-							first = false;
-						} else if (d < minimum) {
-							minimum = d;
-						}
-						i = j;
-						goto break_cont;
-					}
-				}
-			}
-			if (i == loc)
-				neg_flag = false;
-			if (first) {
-				minimum = 1.0;
-				first = false;
-			} else if (1.0 < minimum)
-				minimum = 1.0;
-			i = j;
-			continue;
-		}
-		i++;
-	}
-	if (first || count == 0 || (!neg_flag && minimum == 1.0))
-		return modified;
-	if (!isfinite(minimum))
-		return modified;
-	if (level_code > 1 || (level_code && (level == 1))) {
-		for (i = loc;;) {
-			d = 1.0;
-			if (equation[i].kind == CONSTANT) {
-				if (equation[i].level == level) {
-					d = equation[i].token.constant;
-				} else if ((i + 1) < *np
-				    && equation[i+1].level == (level + 1)
-				    && (equation[i+1].token.operatr == TIMES
-				    || equation[i+1].token.operatr == DIVIDE)) {
-					d = equation[i].token.constant;
-				}
-			}
-			if ((minimum < 1.0 && fmod(d, 1.0) == 0.0)
-			    || (fmod(d / minimum, 1.0) != 0.0)) {
-				if (neg_flag) {
-					minimum = 1.0;
-					break;
-				}
-				return modified;
-			}
-			i++;
-			for (; i < *np && equation[i].level > level; i += 2)
-				;
-			if (i >= *np || equation[i].level < level)
-				break;
-			i++;
-		}
-	}
-	if (neg_flag)
-		minimum = -minimum;
-	if (*np + ((count + 2) * 2) > n_tokens) {
-		error_huge();
-	}
-	for (i = loc; i < *np && equation[i].level >= level; i++) {
-		if (equation[i].kind != OPERATOR) {
-			for (j = i;;) {
-				equation[j].level++;
-				j++;
-				if (j >= *np || equation[j].level <= level)
-					break;
-			}
-			blt(&equation[j+2], &equation[j], (*np - j) * sizeof(*equation));
-			*np += 2;
-			equation[j].level = level + 1;
-			equation[j].kind = OPERATOR;
-			equation[j].token.operatr = DIVIDE;
-			j++;
-			equation[j].level = level + 1;
-			equation[j].kind = CONSTANT;
-			equation[j].token.constant = minimum;
-			i = j;
-		}
-	}
-	for (i = loc; i < *np && equation[i].level >= level; i++) {
-		equation[i].level++;
-	}
-	blt(&equation[i+2], &equation[i], (*np - i) * sizeof(*equation));
-	*np += 2;
-	equation[i].level = level;
-	equation[i].kind = OPERATOR;
-	equation[i].token.operatr = TIMES;
-	i++;
-	equation[i].level = level;
-	equation[i].kind = CONSTANT;
-	equation[i].token.constant = minimum;
-	return true;
-}
