@@ -1,24 +1,26 @@
 /*
- * Algebraic manipulator integration routines.
+ * Algebraic manipulator integration routines and commands.
  *
- * These are very low level, so they don't do much, mostly polynomials.
+ * These are very low level, so they don't do much, polynomials only.
  *
  * Copyright (c) 1987-2005 George Gesslein II.
  */
 
 #include "includes.h"
 
-static int	int_sub();
+static int	integrate_sub();
 static int	laplace_sub();
 static int	inv_laplace_sub();
 
+static double	integrate_order;	/* global order of integration */
+
 /*
  * Make variable "v" always raised to a power,
  * unless it is on the right side of a power operator.
  */
 make_powers(equation, np, v)
-token_type	*equation;	/* pointer to expression */
-int		*np;		/* pointer to length of expression */
+token_type	*equation;	/* equation side pointer */
+int		*np;		/* pointer to length of equation side */
 long		v;		/* variable */
 {
 	int	i;
@@ -56,91 +58,71 @@ long		v;		/* variable */
 
 /*
  * Integration dispatch routine.
+ * Handles the level 1 additive operators.
  *
  * Return true if successful.
  */
 int
 int_dispatch(equation, np, v, func)
-token_type	*equation;
-int		*np;
-long		v;
-int		(*func)();	/* integration function to call */
+token_type	*equation;	/* equation side to integrate */
+int		*np;		/* pointer to length of equation side */
+long		v;		/* integration variable */
+int		(*func)();	/* integration function to call for each term */
 {
 	int	i, j;
 
 	make_powers(equation, np, v);
-	if (level1_plus(equation, *np)) {
-		for (j = 0, i = 1;; i += 2) {
-			if (i >= *np || equation[i].level == 1) {
-				if (!(*func)(equation, np, j, i, v))
-					return false;
-				for (i = j + 1;; i += 2) {
-					if (i >= *np) {
-						return true;
-					}
-					if (equation[i].level == 1) {
-						j = i + 1;
-						break;
-					}
+	for (j = 0, i = 1;; i += 2) {
+		if (i >= *np) {
+			return((*func)(equation, np, j, i, v));
+		}
+		if (equation[i].level == 1
+		    && (equation[i].token.operatr == PLUS || equation[i].token.operatr == MINUS)) {
+			if (!(*func)(equation, np, j, i, v)) {
+				return false;
+			}
+			for (i = j + 1;; i += 2) {
+				if (i >= *np) {
+					return true;
+				}
+				if (equation[i].level == 1) {
+					j = i + 1;
+					break;
 				}
 			}
 		}
-	} else {
-		if (!(*func)(equation, np, 0, *np, v))
-			return false;
 	}
 	return true;
 }
 
 /*
- * Do the actual integration.
+ * Do the actual integration of a polynomial term.
  *
  * Return true if successful.
  */
 static int
-int_sub(equation, np, loc, eloc, v)
-token_type	*equation;
-int		*np;
-int		loc, eloc;
-long		v;
+integrate_sub(equation, np, loc, eloc, v)
+token_type	*equation;	/* equation side pointer */
+int		*np;		/* pointer to length of equation side */
+int		loc;		/* beginning location of term */
+int		eloc;		/* end location of term */
+long		v;		/* variable */
 {
 	int		i, j, k;
 	int		len;
-	int		level, mlevel;
+	int		level, vlevel, mlevel;
 	int		count;
-	int		exp_pos;
-	complexs	c;
 	int		div_flag;
 
-	exp_pos = -1;
 	level = min_level(&equation[loc], eloc - loc);
+	/* determine if the term is a polynomial term in "v" */
 	for (i = loc, count = 0; i < eloc; i += 2) {
 		if (equation[i].kind == VARIABLE && equation[i].token.variable == v) {
 			count++;
 			if (count > 1)
 				return false;
-			if (i > loc && equation[i].level == equation[i-1].level
-			    && equation[i-1].token.operatr == POWER) {
-				exp_pos = i - 1;
-				if (equation[exp_pos].level == level) {
-					continue;
-				} else if (equation[exp_pos].level == level + 1) {
-					for (k = loc + 1; k < eloc; k += 2) {
-						if (equation[k].level == level) {
-							switch (equation[k].token.operatr) {
-							case TIMES:
-							case DIVIDE:
-								break;
-							default:
-								return false;
-							}
-						}
-					}
-					continue;
-				}
-				return false;
-			}
-			if (equation[i].level == level || equation[i].level == (level + 1)) {
+			vlevel = equation[i].level;
+			if (vlevel == level || vlevel == (level + 1)) {
 				for (k = loc + 1; k < eloc; k += 2) {
 					if (equation[k].level == level) {
 						switch (equation[k].token.operatr) {
@@ -155,8 +137,8 @@ long		v;
 						}
 					}
 				}
-				if (equation[i].level == (level + 1)) {
-					if ((i + 1) < eloc && equation[i+1].level == equation[i].level
+				if (vlevel == (level + 1)) {
+					if ((i + 1) < eloc && equation[i+1].level == vlevel
 					    && equation[i+1].token.operatr == POWER) {
 						continue;
 					}
@@ -167,50 +149,6 @@ long		v;
 			return false;
 		}
 	}
-	if (exp_pos >= 0) {
-		for (i = exp_pos - 2; i > loc && equation[i].level > equation[exp_pos].level; i -= 2)
-			;
-		i++;
-		if (!parse_complex(&equation[i], exp_pos - i, &c))
-			return false;
-		c = complex_log(c);
-		if (i > loc && equation[i-1].token.operatr == DIVIDE) {
-			c.re = -c.re;
-			c.im = -c.im;
-		}
-		if (*np + 6 > n_tokens) {
-			error_huge();
-		}
-		for (j = loc; j < eloc; j++)
-			equation[j].level++;
-		blt(&equation[eloc+6], &equation[eloc], (*np - eloc) * sizeof(token_type));
-		*np += 6;
-		k = eloc;
-		equation[k].level = level;
-		equation[k].kind = OPERATOR;
-		equation[k].token.operatr = DIVIDE;
-		k++;
-		equation[k].level = level + 1;
-		equation[k].kind = CONSTANT;
-		equation[k].token.constant = c.re;
-		k++;
-		equation[k].level = level + 1;
-		equation[k].kind = OPERATOR;
-		equation[k].token.operatr = PLUS;
-		k++;
-		equation[k].level = level + 2;
-		equation[k].kind = CONSTANT;
-		equation[k].token.constant = c.im;
-		k++;
-		equation[k].level = level + 2;
-		equation[k].kind = OPERATOR;
-		equation[k].token.operatr = TIMES;
-		k++;
-		equation[k].level = level + 2;
-		equation[k].kind = VARIABLE;
-		equation[k].token.variable = IMAGINARY;
-		return true;
-	}
 	mlevel = level + 1;
 	for (j = loc; j < eloc; j++)
 		equation[j].level += 2;
@@ -296,13 +234,13 @@ char	*cp;
 {
 	int		i, j;
 	long		v;
-	double		d1, order;
+	double		d1;
 	token_type	*source, *dest;
 	int		n1, n2, *nps, *np;
 	int		def_flag;
 
-	order = 1.0;
-	if (notdefined(cur_equation)) {
+	integrate_order = 1.0;
+	if (current_not_defined()) {
 		return false;
 	}
 	if (def_flag = (strcmp_tospace(cp, "definite") == 0)) {
@@ -326,9 +264,9 @@ char	*cp;
 			return false;
 		}
 		if (*cp) {
-			order = strtod(cp, &cp);
+			integrate_order = strtod(cp, &cp);
 		}
-		if (*cp || order <= 0 || fmod(order, 1.0) != 0.0) {
+		if (*cp || integrate_order <= 0 || fmod(integrate_order, 1.0) != 0.0) {
 			error(_("The order must be a positive integer."));
 			return false;
 		}
@@ -343,8 +281,8 @@ char	*cp;
 	factorv(source, nps, v);
 	blt(dest, source, *nps * sizeof(token_type));
 	n1 = *nps;
-	for (d1 = 0; d1 < order; d1++) {
-		if (!int_dispatch(dest, &n1, v, int_sub)) {
+	for (d1 = 0; d1 < integrate_order; d1++) {
+		if (!int_dispatch(dest, &n1, v, integrate_sub)) {
 			error(_("Integration failed."));
 			return false;
 		}
@@ -379,7 +317,7 @@ char	*cp;
 		blt(&dest[n1], scratch, n2 * sizeof(token_type));
 		n1 += n2;
 	}
-	simpa_side(dest, &n1, false, false);
+	simpa_side(dest, &n1, false);
 	if (n_rhs[cur_equation]) {
 		blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
 		n_lhs[i] = n_lhs[cur_equation];
@@ -390,6 +328,11 @@ char	*cp;
 	return true;
 }
 
+/*
+ * Do the actual Laplace transformation of a polynomial term.
+ *
+ * Return true if successful.
+ */
 static int
 laplace_sub(equation, np, loc, eloc, v)
 token_type	*equation;
@@ -399,120 +342,9 @@ long		v;
 {
 	int		i, j, k;
 	int		len;
-	int		level, level2, mlevel;
-	int		count;
-	int		exp_pos;
-	complexs	c;
+	int		level, mlevel;
 
-	exp_pos = -1;
-	level = min_level(&equation[loc], eloc - loc);
-	for (i = loc, count = 0; i < eloc; i += 2) {
-		if (equation[i].kind == VARIABLE && equation[i].token.variable == v) {
-			count++;
-			if (count > 1)
-				return false;
-			if (i > loc && equation[i].level == equation[i-1].level
-			    && equation[i-1].token.operatr == POWER) {
-				if (equation[i-2].level != equation[i-1].level) {
-					return false;
-				}
-				exp_pos = i - 2;
-				if (equation[i].level == level) {
-					continue;
-				} else if (equation[i].level == level + 1) {
-					for (k = loc + 1; k < eloc; k += 2) {
-						if (equation[k].level == level) {
-							switch (equation[k].token.operatr) {
-							case TIMES:
-							case DIVIDE:
-								break;
-							default:
-								return false;
-							}
-						}
-					}
-					continue;
-				}
-				return false;
-			}
-			if (equation[i].level == level || equation[i].level == (level + 1)) {
-				for (k = loc + 1; k < eloc; k += 2) {
-					if (equation[k].level == level) {
-						switch (equation[k].token.operatr) {
-						case DIVIDE:
-							if (k == (i - 1))
-								return false;
-						case TIMES:
-							continue;
-						case POWER:
-							if (k == (i + 1))
-								continue;
-						default:
-							return false;
-						}
-					}
-				}
-				if (equation[i].level == (level + 1)) {
-					if ((i + 1) < eloc && equation[i+1].level == equation[i].level
-					    && equation[i+1].token.operatr == POWER) {
-						continue;
-					}
-				} else {
-					continue;
-				}
-			}
-			return false;
-		}
-	}
-	if (exp_pos >= 0) {
-		if (*np + 6 > n_tokens) {
-			error_huge();
-		}
-		if (!parse_complex(&equation[exp_pos], 1, &c))
-			return false;
-		c = complex_log(c);
-		if (exp_pos > loc && equation[exp_pos-1].token.operatr == DIVIDE) {
-			equation[exp_pos-1].token.operatr = TIMES;
-			c.re = -c.re;
-			c.im = -c.im;
-		}
-		equation[exp_pos].kind = CONSTANT;
-		equation[exp_pos].token.constant = 1.0;
-		exp_pos++;
-		level2 = equation[exp_pos].level + 1;
-		equation[exp_pos].token.operatr = DIVIDE;
-		exp_pos++;
-		equation[exp_pos].level = level2;
-		exp_pos++;
-		blt(&equation[exp_pos+6], &equation[exp_pos], (*np - exp_pos) * sizeof(token_type));
-		*np += 6;
-		k = exp_pos;
-		equation[k].level = level2;
-		equation[k].kind = OPERATOR;
-		equation[k].token.operatr = MINUS;
-		k++;
-		equation[k].level = level2 + 1;
-		equation[k].kind = CONSTANT;
-		equation[k].token.constant = c.re;
-		k++;
-		equation[k].level = level2 + 1;
-		equation[k].kind = OPERATOR;
-		equation[k].token.operatr = PLUS;
-		k++;
-		equation[k].level = level2 + 2;
-		equation[k].kind = CONSTANT;
-		equation[k].token.constant = c.im;
-		k++;
-		equation[k].level = level2 + 2;
-		equation[k].kind = OPERATOR;
-		equation[k].token.operatr = TIMES;
-		k++;
-		equation[k].level = level2 + 2;
-		equation[k].kind = VARIABLE;
-		equation[k].token.variable = IMAGINARY;
-		return true;
-	}
-	mlevel = level + 1;
+	mlevel = min_level(&equation[loc], eloc - loc) + 1;
 	for (j = loc; j < eloc; j++)
 		equation[j].level += 2;
 	for (i = loc; i < eloc; i += 2) {
@@ -585,6 +417,11 @@ long		v;
 	return true;
 }
 
+/*
+ * Do the actual inverse Laplace transformation of a polynomial term.
+ *
+ * Return true if successful.
+ */
 static int
 inv_laplace_sub(equation, np, loc, eloc, v)
 token_type	*equation;
@@ -595,16 +432,7 @@ long		v;
 	int	i, j, k;
 	int	len;
 	int	level, mlevel;
-	int	count;
 
-	count = 0;
-	for (i = loc; i < eloc; i += 2) {
-		if (equation[i].kind == VARIABLE && equation[i].token.variable == v) {
-			count++;
-		}
-	}
-	if (count > 1)
-		return false;
 	mlevel = min_level(&equation[loc], eloc - loc) + 1;
 	for (j = loc; j < eloc; j++)
 		equation[j].level += 2;
@@ -672,7 +500,7 @@ char	*cp;
 	token_type	*source, *dest;
 	int		n1, *nps, *np;
 
-	if (notdefined(cur_equation)) {
+	if (current_not_defined()) {
 		return false;
 	}
 	i = next_espace();
@@ -717,7 +545,8 @@ char	*cp;
 			return false;
 		}
 	} else {
-		if (!int_dispatch(dest, &n1, v, laplace_sub)) {
+		if (!poly_in_v(dest, n1, v, false)
+		    || !int_dispatch(dest, &n1, v, laplace_sub)) {
 			error(_("Laplace failed."));
 			return false;
 		}
@@ -733,7 +562,7 @@ char	*cp;
 	return true;
 }
 
-#if	!BASICS
+#if	!BASICS && !LIBRARY
 /*
  * Numerical integrate command.
  */
@@ -742,20 +571,17 @@ nintegrate_cmd(cp)
 char	*cp;
 {
 	long		v;
-	int		i, j, k;
-	int		i1, j1;
+	int		i, j, k, i1, i2;
 	int		level;
 	int		iterations;
 	int		first_size;
-	token_type	*ep;
-	int		trap_flag;
-	int		singularity;
-	token_type	*source, *dest;
+	int		trap_flag, singularity;
+	token_type	*ep, *source, *dest;
 	int		n1, *nps, *np;
 
 	iterations = 1000;	/* must be even */
 	first_size = 0;
-	if (notdefined(cur_equation)) {
+	if (current_not_defined()) {
 		return false;
 	}
 	i = next_espace();
@@ -774,10 +600,7 @@ char	*cp;
 	if (trap_flag) {
 		cp = skip_param(cp);
 	}
-	if (*cp == '\0') {
-		if (!prompt_var(&v))
-			return false;
-	} else {
+	if (*cp) {
 		cp = parse_var2(&v, cp);
 		if (cp == NULL) {
 			return false;
@@ -789,14 +612,15 @@ char	*cp;
 			error(_("Number of partitions must be a positive, even integer."));
 			return false;
 		}
+	} else {
+		if (!prompt_var(&v))
+			return false;
 	}
 	singularity = false;
 	for (j = 1; j < *nps; j += 2) {
 		if (source[j].token.operatr == DIVIDE) {
-			level = source[j].level;
-			for (k = j + 1; k < *nps && source[k].level >= level; k++) {
-				if (source[k].kind == VARIABLE
-				    && source[k].token.variable == v) {
+			for (k = j + 1; k < *nps && source[k].level >= source[j].level; k++) {
+				if (source[k].kind == VARIABLE && source[k].token.variable == v) {
 					singularity = true;
 				}
 			}
@@ -810,22 +634,18 @@ char	*cp;
 		return false;
 	}
 	subst_constants(tlhs, &n_tlhs);
-	for (j = 0; j < n_tlhs; j += 2) {
-		if (tlhs[j].kind == CONSTANT && !isfinite(tlhs[j].token.constant)) {
-			error(_("Error: Bound contains infinity."));
-			return false;
-		}
+	if (exp_contains_infinity(tlhs, n_tlhs)) {
+		error(_("Error: Bound contains infinity."));
+		return false;
 	}
 	my_strlcpy(prompt_str, _("Enter upper bound: "), sizeof(prompt_str));
 	if (!get_expr(trhs, &n_trhs)) {
 		return false;
 	}
 	subst_constants(trhs, &n_trhs);
-	for (j = 0; j < n_trhs; j += 2) {
-		if (trhs[j].kind == CONSTANT && !isfinite(trhs[j].token.constant)) {
-			error(_("Error: Bound contains infinity."));
-			return false;
-		}
+	if (exp_contains_infinity(trhs, n_trhs)) {
+		error(_("Error: Bound contains infinity."));
+		return false;
 	}
 	if ((n_tlhs + 1 + n_trhs + 2) > n_tokens) {
 		error_huge();
@@ -837,8 +657,9 @@ char	*cp;
 	}
 	subst_constants(source, nps);
 	simp_loop(source, nps);
-	for (j = 0; j < n_trhs; j++)
+	for (j = 0; j < n_trhs; j++) {
 		trhs[j].level += 2;
+	}
 	trhs[n_trhs].level = 2;
 	trhs[n_trhs].kind = OPERATOR;
 	trhs[n_trhs].token.operatr = MINUS;
@@ -846,8 +667,9 @@ char	*cp;
 	j = n_trhs;
 	blt(&trhs[n_trhs], tlhs, n_tlhs * sizeof(token_type));
 	n_trhs += n_tlhs;
-	for (; j < n_trhs; j++)
+	for (; j < n_trhs; j++) {
 		trhs[j].level += 2;
+	}
 	trhs[n_trhs].level = 1;
 	trhs[n_trhs].kind = OPERATOR;
 	trhs[n_trhs].token.operatr = DIVIDE;
@@ -857,15 +679,14 @@ char	*cp;
 	trhs[n_trhs].token.constant = iterations;
 	n_trhs++;
 	simp_loop(trhs, &n_trhs);
-	dest[0].level = 1;
-	dest[0].kind = CONSTANT;
-	dest[0].token.constant = 0.0;
+	dest[0] = zero_token;
 	n1 = 1;
 	for (j = 0; j <= iterations; j++) {
 		if ((n1 + 1 + *nps) > n_tokens)
 			error_huge();
-		for (k = 0; k < n1; k++)
+		for (k = 0; k < n1; k++) {
 			dest[k].level++;
+		}
 		ep = &dest[n1];
 		ep->level = 1;
 		ep->kind = OPERATOR;
@@ -874,40 +695,44 @@ char	*cp;
 		i1 = n1;
 		blt(&dest[i1], source, *nps * sizeof(token_type));
 		n1 += *nps;
-		for (k = i1; k < n1; k++)
+		for (k = i1; k < n1; k++) {
 			dest[k].level += 2;
+		}
 		for (k = i1; k < n1; k += 2) {
-			if (dest[k].kind == VARIABLE
-			    && dest[k].token.variable == v) {
+			if (dest[k].kind == VARIABLE && dest[k].token.variable == v) {
 				level = dest[k].level;
-				j1 = n_tlhs + 2 + n_trhs;
-				if ((n1 + j1) > n_tokens)
+				i2 = n_tlhs + 2 + n_trhs;
+				if ((n1 + i2) > n_tokens)
 					error_huge();
-				blt(&dest[k+1+j1], &dest[k+1], (n1 - (k + 1)) * sizeof(token_type));
-				n1 += j1;
-				j1 = k;
+				blt(&dest[k+1+i2], &dest[k+1], (n1 - (k + 1)) * sizeof(token_type));
+				n1 += i2;
+				i2 = k;
 				blt(&dest[k], tlhs, n_tlhs * sizeof(token_type));
 				k += n_tlhs;
-				for (; j1 < k; j1++)
-					dest[j1].level += level + 1;
+				level++;
+				for (; i2 < k; i2++) {
+					dest[i2].level += level;
+				}
 				ep = &dest[k];
-				ep->level = level + 1;
+				ep->level = level;
 				ep->kind = OPERATOR;
 				ep->token.operatr = PLUS;
 				ep++;
-				ep->level = level + 2;
+				level++;
+				ep->level = level;
 				ep->kind = CONSTANT;
 				ep->token.constant = j;
 				ep++;
-				ep->level = level + 2;
+				ep->level = level;
 				ep->kind = OPERATOR;
 				ep->token.operatr = TIMES;
 				k += 3;
-				j1 = k;
+				i2 = k;
 				blt(&dest[k], trhs, n_trhs * sizeof(token_type));
 				k += n_trhs;
-				for (; j1 < k; j1++)
-					dest[j1].level += level + 2;
+				for (; i2 < k; i2++) {
+					dest[i2].level += level;
+				}
 				k--;
 			}
 		}
@@ -932,28 +757,31 @@ char	*cp;
 			}
 			n1 += 2;
 		}
+
 		in_calc_cmd = true;
 		elim_loop(dest, &n1);
 		ufactor(dest, &n1);
 		simp_divide(dest, &n1);
 		in_calc_cmd = false;
-		for (k = 0; k < n1; k += 2) {
-			if (dest[k].kind == CONSTANT && !isfinite(dest[k].token.constant)) {
-				error(_("Integration failed because result contains infinity or nan."));
+
+		if (exp_contains_infinity(dest, n1)) {
+			error(_("Integration failed because result contains infinity or nan."));
+			return false;
+		}
+		switch (j) {
+		case 0:
+			break;
+		case 1:
+			first_size = n1;
+			if (first_size < 4)
+				first_size = 4;
+			break;
+		default:
+			if ((n1 / 8) >= first_size) {
+				error(_("Integration failed."));
 				return false;
 			}
-		}
-		if (j > 0) {
-			if (j == 1) {
-				first_size = n1;
-				if (first_size < 4)
-					first_size = 4;
-			} else {
-				if ((n1 / 8) >= first_size) {
-					error(_("Integration failed."));
-					return false;
-				}
-			}
+			break;
 		}
 	}
 	if ((n1 + 3 + n_trhs) > n_tokens)
@@ -982,9 +810,13 @@ char	*cp;
 	n1 += n_trhs;
 	for (; k < n1; k++)
 		dest[k].level++;
+
+	in_calc_cmd = true;
 	elim_loop(dest, &n1);
 	ufactor(dest, &n1);
 	simp_divide(dest, &n1);
+	in_calc_cmd = false;
+
 	debug_string(0, _("Integration successful."));
 	if (n_rhs[cur_equation]) {
 		blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
