@@ -11,7 +11,6 @@
 
 #include "../includes.h"
 #include "mathomatic.h"
-#include <signal.h>
 
 /*
  * Initialize Mathomatic.
@@ -80,12 +79,13 @@ matho_process(char *input, char **outputp)
 }
 
 /*
- * Process a Mathomatic equation or expression.
- * Input string is in "input", output string is stored in "*outputp".
- * Same as above, except commands are not allowed, so that variables
- * are not confused with commands.
+ * Parse an equation or expression and store in the next available equation space,
+ * making it the current equation.
  *
- * Use this when entering or solving equations.
+ * Input string is in "input", output string is stored in "*outputp".
+ *
+ * Same as matho_process() above, except commands are not allowed, so that variables
+ * are not confused with commands.
  *
  * Returns true if successful.
  */
@@ -123,6 +123,11 @@ matho_parse(char *input, char **outputp)
 	return rv;
 }
 
+/*
+ * Floating point exception handler.
+ *
+ * Usually doesn't work in most operating systems.
+ */
 void
 fphandler(int sig)
 {
