@@ -54,8 +54,9 @@ int	color;
 		return;
 	if (color_flag) {
 		if (html_flag) {
-			if (cur_color >= 0)
+			if (cur_color >= 0) {
 				printf("</font>");
+			}
 			printf("<font color=\"%s\">", html_carray[color%ARR_CNT(html_carray)]);
 		} else {
 			if (bold_colors) {
@@ -77,7 +78,7 @@ default_color()
 }
 
 /*
- * Display the equation stored in equation space "n" in single line format.
+ * Display the expression or equation stored in equation space "n" in single line format.
  *
  * Return true if successful.
  */
@@ -113,7 +114,7 @@ int	export_flag;	/* true for exportable format (readable by other math programs)
 }
 
 /*
- * Display the equation stored in equation space "n".
+ * Display the expression or equation stored in equation space "n".
  *
  * Return true if successful.
  */
@@ -125,8 +126,16 @@ int	n;	/* equation number */
 		return false;
 	if (groupall) {
 		group_sub(n);
+		if (factor_int_flag) {
+			factor_int(lhs[n], &n_lhs[n]);
+			factor_int(rhs[n], &n_rhs[n]);
+		}
 		return flist_sub(n);
 	} else {
+		if (factor_int_flag) {
+			factor_int(lhs[n], &n_lhs[n]);
+			factor_int(rhs[n], &n_rhs[n]);
+		}
 		return list1_sub(n, false);
 	}
 }
@@ -140,7 +149,7 @@ token_type	*p2;
 int		n2;
 {
 	if (debug_level >= level) {
-		printf(_("debug %d: "), level);
+		printf(_("level %d: "), level);
 		list_proc(p1, n1, false);
 		if (p2 && n2 > 0) {
 			printf(" = ");
@@ -154,19 +163,21 @@ int		n2;
 /*
  * Convert variable "v" to an ASCII variable name.
  * The ASCII variable name is stored in global "var_str[]".
- * Make it C language compatible if "c_flag" is true.
  *
  * Return length of variable name (number of ASCII characters).
  *
- * If "c_flag == 2", assume Java.  If "c_flag" is negative, create
- * an exportable variable name: -1 for Maxima, -2 for other.
+ * If "lang_code == 0", use standard Mathomatic format.
+ * If "lang_code == 1", make variable name C language compatible.
+ * If "lang_code == 2", assume Java.
+ * If "lang_code == 3", assume Python.
+ * If "lang_code" is negative, create an exportable variable name: -1 for Maxima, -2 for other.
  */
 int
-list_var(v, c_flag)
-long	v;
-int	c_flag;
+list_var(v, lang_code)
+long	v;		/* variable to convert */
+int	lang_code;	/* language code */
 {
-	int		i, j;
+	int		j;
 	char		*cp, buf[100];
 
 	cp = NULL;
@@ -178,7 +189,7 @@ int	c_flag;
 		cp = "sign";
 		break;
 	case IMAGINARY:
-		switch (c_flag) {
+		switch (lang_code) {
 		case -2:
 			cp = "i";
 			break;
@@ -188,13 +199,16 @@ int	c_flag;
 		case 0:
 			cp = "i#";
 			break;
+		case 3:
+			cp = "1j";
+			break;
 		default:
 			cp = "I";
 			break;
 		}
 		break;
 	case V_E:
-		switch (c_flag) {
+		switch (lang_code) {
 		case -2:
 			cp = "e";
 			break;
@@ -204,29 +218,34 @@ int	c_flag;
 		case 0:
 			cp = "e#";
 			break;
-		case 1:
-			cp = "E";
-			break;
 		case 2:
 			cp = "Math.E";
 			break;
+		case 3:
+			cp = "math.e";
+			break;
+		default:
+			cp = "E";
+			break;
 		}
 		break;
 	case V_PI:
-		switch (c_flag) {
+		switch (lang_code) {
 		case -1:
 			cp = "%pi";
 			break;
-		case 0:
-		case -2:
-			cp = "pi";
-			break;
 		case 1:
 			cp = "PI";
 			break;
 		case 2:
 			cp = "Math.PI";
 			break;
+		case 3:
+			cp = "math.pi";
+			break;
+		default:
+			cp = "pi";
+			break;
 		}
 		break;
 	case MATCH_ANY:
@@ -256,9 +275,9 @@ int	c_flag;
 	j = (v >> VAR_SHIFT) & SUBSCRIPT_MASK;
 	if (j) {
 		snprintf(buf, sizeof(buf), "%d", j - 1);
-		my_strlcat(var_str, buf, sizeof(var_str));
+		strcat(var_str, buf);
 	}
-	if (c_flag > 0) {
+	if (lang_code > 0) {
 		for (cp = var_str; *cp; cp++) {
 			if (*cp == '\\') {
 				*cp = '_';
@@ -285,9 +304,8 @@ int		export_flag;	/* flag for exportable format (usually false) */
 	int	min1;
 	int	cur_level;
 	char	*cp;
-	int	len;
+	int	len = 0;
 
-	len = 0;
 	cur_level = min1 = min_level(equation, n);
 	for (i = 0; i < n; i++) {
 		j = cur_level - equation[i].level;
@@ -306,7 +324,11 @@ int		export_flag;	/* flag for exportable format (usually false) */
 		switch (equation[i].kind) {
 		case CONSTANT:
 			if (high_prec || export_flag) {
-				len += fprintf(gfp, "%.20g", equation[i].token.constant);
+				if (equation[i].token.constant <= 0.0) {
+					len += fprintf(gfp, "(%.20g)", equation[i].token.constant);
+				} else {
+					len += fprintf(gfp, "%.20g", equation[i].token.constant);
+				}
 			} else if (finance_option) {
 				len += fprintf(gfp, "%.2f", equation[i].token.constant);
 			} else {
@@ -336,7 +358,11 @@ int		export_flag;	/* flag for exportable format (usually false) */
 				cp = " % ";
 				break;
 			case POWER:
-				cp = "^";
+				if (starstar) {
+					cp = "**";
+				} else {
+					cp = "^";
+				}
 				break;
 			case FACTORIAL:
 				cp = "!";
@@ -364,7 +390,7 @@ int		export_flag;	/* flag for exportable format (usually false) */
 char *
 list_equation(n, export_flag)
 int	n;		/* equation number */
-int	export_flag;
+int	export_flag;	/* flag for exportable format (usually false) */
 {
 	int	len;
 	char	*cp;
@@ -433,12 +459,11 @@ int		export_flag;
 	int	min1;
 	int	cur_level;
 	char	*cp;
-	int	len;
+	int	len = 0;
 	char	buf[500];
 
 	if (string)
 		string[0] = '\0';
-	len = 0;
 	cur_level = min1 = min_level(equation, n);
 	for (i = 0; i < n; i++) {
 		j = cur_level - equation[i].level;
@@ -459,7 +484,11 @@ int		export_flag;
 		switch (equation[i].kind) {
 		case CONSTANT:
 			if (high_prec || export_flag) {
-				len += snprintf(buf, sizeof(buf), "%.20g", equation[i].token.constant);
+				if (equation[i].token.constant <= 0.0) {
+					len += snprintf(buf, sizeof(buf), "(%.20g)", equation[i].token.constant);
+				} else {
+					len += snprintf(buf, sizeof(buf), "%.20g", equation[i].token.constant);
+				}
 			} else if (finance_option) {
 				len += snprintf(buf, sizeof(buf), "%.2f", equation[i].token.constant);
 			} else {
@@ -493,7 +522,11 @@ int		export_flag;
 				cp = " % ";
 				break;
 			case POWER:
-				cp = "^";
+				if (starstar) {
+					cp = "**";
+				} else {
+					cp = "^";
+				}
 				break;
 			case FACTORIAL:
 				cp = "!";
@@ -514,10 +547,9 @@ int		export_flag;
 	return len;
 }
 
-#if	!BASICS
 /*
  * Return true if expression is a valid integer expression for
- * list_c_code().
+ * list_code().
  */
 int
 int_expr(equation, n)
@@ -525,33 +557,21 @@ token_type	*equation;	/* expression pointer */
 int		n;		/* length of expression */
 {
 	int	i;
+	double	d;
 
 	for (i = 0; i < n; i++) {
-		switch (equation[i].kind) {
-		case CONSTANT:
-			if (fmod(equation[i].token.constant, 1.0)) {
-				return false;
-			}
-			break;
-		case OPERATOR:
-			switch (equation[i].token.operatr) {
-			case POWER:
-				if (equation[i+1].level == equation[i].level
-				    && equation[i-1].level == equation[i].level
-				    && equation[i+1].kind == CONSTANT
-				    && equation[i+1].token.constant == 2.0) {
-					continue;
-				}
-				return false;
-			}
-			break;
+		if ((equation[i].kind == CONSTANT
+		    && fmod(equation[i].token.constant, 1.0) != 0.0)
+		    || (equation[i].kind == VARIABLE
+		    && var_is_const(equation[i].token.variable, &d))) {
+			return false;	/* non-integer constant */
 		}
 	}
 	return true;
 }
 
 /*
- * Display an equation as C or Java code.
+ * Display an equation as C, Java, or Python code.
  */
 list_c_equation(en, java_flag, int_flag)
 int	en;		/* equation number */
@@ -560,39 +580,39 @@ int	int_flag;	/* integer arithmetic flag */
 {
 	if (n_lhs[en] <= 0)
 		return;
-	list_c_code(lhs[en], n_lhs[en], java_flag, int_flag);
+	list_code(lhs[en], &n_lhs[en], java_flag, int_flag);
 	if (n_rhs[en]) {
 		fprintf(gfp, " = ");
-		list_c_code(rhs[en], n_rhs[en], java_flag, int_flag);
+		list_code(rhs[en], &n_rhs[en], java_flag, int_flag);
 	}
 	fprintf(gfp, ";\n");
 }
 
 /*
- * Display C or Java code for an expression.
+ * Output C, Java, or Python code for an expression.
+ * Expression may be modified.
  *
  * Return length of output (number of characters).
  */
 int
-list_c_code(equation, n, java_flag, int_flag)
+list_code(equation, np, java_flag, int_flag)
 token_type	*equation;	/* expression pointer */
-int		n;		/* length of expression */
-int		java_flag;	/* if true, generate jave code */
-int		int_flag;	/* integer arithmetic flag */
+int		*np;		/* pointer to length of expression */
+int		java_flag;	/* if 1, generate Java code, if 2, generate Python code */
+int		int_flag;	/* integer arithmetic flag, should work with any language */
 {
 	int	i, j, k, i1, i2;
 	int	min1;
 	int	cur_level;
 	char	*cp;
-	char	buf[50];
-	int	len;
+	char	buf[500];
+	int	len = 0;
 
-	len = 0;
-	min1 = min_level(equation, n);
-	if (n > 1)
+	min1 = min_level(equation, *np);
+	if (*np > 1)
 		min1--;
 	cur_level = min1;
-	for (i = 0; i < n; i++) {
+	for (i = 0; i < *np; i++) {
 		j = cur_level - equation[i].level;
 		k = abs(j);
 		for (i1 = 1; i1 <= k; i1++) {
@@ -602,7 +622,7 @@ int		int_flag;	/* integer arithmetic flag */
 				len++;
 			} else {
 				cur_level++;
-				for (i2 = i + 1; i2 < n && equation[i2].level >= cur_level; i2 += 2) {
+				for (i2 = i + 1; i2 < *np && equation[i2].level >= cur_level; i2 += 2) {
 					if (equation[i2].level == cur_level) {
 						switch (equation[i2].token.operatr) {
 						case POWER:
@@ -611,9 +631,10 @@ int		int_flag;	/* integer arithmetic flag */
 							    && equation[i2+1].kind == CONSTANT
 							    && equation[i2+1].token.constant == 2.0) {
 								equation[i2].token.operatr = TIMES;
-								equation[i2+1].kind = equation[i2-1].kind;
-								equation[i2+1].token = equation[i2-1].token;
+								equation[i2+1] = equation[i2-1];
 							} else {
+								if (int_flag || java_flag > 1)
+									break;
 								if (java_flag) {
 									cp = "Math.pow";
 								} else {
@@ -623,11 +644,7 @@ int		int_flag;	/* integer arithmetic flag */
 							}
 							break;
 						case FACTORIAL:
-							if (java_flag) {
-								cp = "MathSupport.fact";
-							} else {
-								cp = "fact";
-							}
+							cp = "fact";
 							len += fprintf(gfp, cp);
 							break;
 						}
@@ -641,23 +658,24 @@ int		int_flag;	/* integer arithmetic flag */
 		switch (equation[i].kind) {
 		case CONSTANT:
 			if (int_flag) {
-				len += fprintf(gfp, "%.0f", equation[i].token.constant);
+				snprintf(buf, sizeof(buf), "%.0f", equation[i].token.constant);
 			} else {
 				snprintf(buf, sizeof(buf), "%#.16g", equation[i].token.constant);
 				j = strlen(buf) - 1;
 				for (; j >= 0; j--) {
 					if (buf[j] == '0')
 						continue;
-					if (buf[j] == '.')
+					if (buf[j] == '.') {
 						buf[j+2] = '\0';
-					else
+					} else {
 						break;
+					}
 				}
-				len += fprintf(gfp, "%s", buf);
 			}
+			len += fprintf(gfp, "%s", buf);
 			break;
 		case VARIABLE:
-			len += list_var(equation[i].token.variable, (java_flag ? 2 : 1));
+			len += list_var(equation[i].token.variable, java_flag + 1);
 			fprintf(gfp, "%s", var_str);
 			break;
 		case OPERATOR:
@@ -679,7 +697,11 @@ int		int_flag;	/* integer arithmetic flag */
 				cp = " % ";
 				break;
 			case POWER:
-				cp = ", ";
+				if (int_flag || java_flag > 1) {
+					cp = " ** ";
+				} else {
+					cp = ", ";
+				}
 				break;
 			case FACTORIAL:
 				cp = "";
@@ -695,13 +717,12 @@ int		int_flag;	/* integer arithmetic flag */
 	}
 	return len;
 }
-#endif
 
-int	cur_line;
-int	cur_pos;
+int	cur_line;	/* current line */
+int	cur_pos;	/* current position in the current line on the screen */
 
 /*
- * Display an equation in fraction format.
+ * Display an expression or equation in multi-line fraction format.
  * Use color if available.
  *
  * Return true if successful.
@@ -828,12 +849,11 @@ int		*highp, *lowp;
 	int	level;
 	int	start_level;
 	int	oflag;
-	int	len, len1, len2;
+	int	len = 0, len1, len2;
 	int	high, low;
 	char	buf[500];
 	char	*cp;
 
-	len = 0;
 	start_level = cur_level;
 	*highp = line;
 	*lowp = line;
@@ -951,7 +971,7 @@ check_again:
 				fprintf(gfp, "%s", buf);
 			break;
 		case VARIABLE:
-			len += list_var(equation[i].token.variable, false);
+			len += list_var(equation[i].token.variable, 0);
 			if (oflag)
 				fprintf(gfp, "%s", var_str);
 			break;
@@ -974,7 +994,11 @@ check_again:
 				cp = " % ";
 				break;
 			case POWER:
-				cp = "^";
+				if (starstar) {
+					cp = "**";
+				} else {
+					cp = "^";
+				}
 				break;
 			case FACTORIAL:
 				cp = "!";
