@@ -7,6 +7,8 @@
 
 #include "includes.h"
 
+#define	REMAINDER_IS_ZERO()	(n_trhs == 1 && trhs[0].kind == CONSTANT && trhs[0].token.constant == 0.0)
+
 /* For smart_div() and poly_div(): */
 token_type	divisor[DIVISOR_SIZE];
 int		n_divisor;
@@ -16,7 +18,6 @@ int		n_quotient;
 token_type	gcd_divisor[DIVISOR_SIZE];
 int		len_d;
 
-static int	poly_in_v_sub();
 static int	do_gcd();
 static int	pf_recurse();
 static int	pf_sub();
@@ -26,7 +27,7 @@ static int	pdiv_recurse();
 static int	poly_div_sub();
 
 /*
- * Compare function for qsort().
+ * Compare function for qsort(3).
  */
 static int
 vcmp(p1, p2)
@@ -36,45 +37,9 @@ sort_type	*p1, *p2;
 }
 
 /*
- * Return true if passed expression is a polynomial in
- * variable v.  The exponents may be anything, as long
- * as they don't contain the variable.
+ * Return true if passed expression is strictly a single polynomial term in v.
  */
-int
-poly_in_v(equation, n, v, allow_divides)
-token_type	*equation;	/* expression pointer */
-int		n;		/* expression length */
-long		v;		/* variable */
-int		allow_divides;	/* allow variable to be right of a divide (negative exponents) */
-{
-	int	i, j;
-	int	rv;
-
-	rv = true;
-	if (level1_plus(equation, n)) {
-		for (i = 1, j = 0;; i += 2) {
-			if (i >= n || equation[i].level == 1) {
-				if (!poly_in_v_sub(&equation[j], i - j, v, allow_divides)) {
-					rv = false;
-					break;
-				}
-				j = i + 1;
-			}
-			if (i >= n)
-				break;
-		}
-	} else {
-		if (!poly_in_v_sub(equation, n, v, allow_divides))
-			rv = false;
-	}
-	return rv;
-}
-
-/*
- * Return true if passed expression is strictly a single
- * polynomial term in v.
- */
-static int
+static inline int
 poly_in_v_sub(equation, n, v, allow_divides)
 token_type	*equation;
 int		n;
@@ -82,7 +47,7 @@ long		v;
 int		allow_divides;
 {
 	int	i, k;
-	int	level;
+	int	level, vlevel;
 	int	count;
 
 	level = min_level(equation, n);
@@ -91,7 +56,8 @@ int		allow_divides;
 			count++;
 			if (count > 1)
 				return false;
-			if (equation[i].level == level || equation[i].level == (level + 1)) {
+			vlevel = equation[i].level;
+			if (vlevel == level || vlevel == (level + 1)) {
 				for (k = 1; k < n; k += 2) {
 					if (equation[k].level == level) {
 						switch (equation[k].token.operatr) {
@@ -108,8 +74,8 @@ int		allow_divides;
 						}
 					}
 				}
-				if (equation[i].level == (level + 1)) {
-					if ((i + 1) < n && equation[i+1].level == equation[i].level
+				if (vlevel == (level + 1)) {
+					if ((i + 1) < n && equation[i+1].level == vlevel
 					    && equation[i+1].token.operatr == POWER) {
 						continue;
 					}
@@ -123,6 +89,34 @@ int		allow_divides;
 	return true;
 }
 
+/*
+ * Return true if passed expression is a polynomial in
+ * variable v.  The exponents may be anything, as long
+ * as they don't contain the variable.
+ */
+int
+poly_in_v(equation, n, v, allow_divides)
+token_type	*equation;	/* expression pointer */
+int		n;		/* expression length */
+long		v;		/* variable */
+int		allow_divides;	/* allow variable to be right of a divide (negative exponents) */
+{
+	int	i, j;
+
+	for (i = 1, j = 0;; i += 2) {
+		if (i >= n || (equation[i].level == 1
+		    && (equation[i].token.operatr == PLUS || equation[i].token.operatr == MINUS))) {
+			if (!poly_in_v_sub(&equation[j], i - j, v, allow_divides)) {
+				return false;
+			}
+			j = i + 1;
+		}
+		if (i >= n)
+			break;
+	}
+	return true;
+}
+
 /*
  * Factor polynomials.
  * Factors repeated factor polynomials (like (x+1)^5)
@@ -143,12 +137,11 @@ pf_recurse(equation, np, loc, level)
 token_type	*equation;
 int		*np, loc, level;
 {
-	int	modified;
+	int	modified = false;
 	int	i;
 	int	op;
 	int	len;
 
-	modified = false;
 	op = 0;
 	for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
 		if (equation[i].level == level) {
@@ -162,8 +155,6 @@ int		*np, loc, level;
 		modified = pf_sub(equation, np, loc, len, level);
 		break;
 	}
-	if (modified)
-		return true;
 	for (i = loc; i < *np && equation[i].level >= level;) {
 		if (equation[i].level > level) {
 			modified |= pf_recurse(equation, np, i, level + 1);
@@ -189,30 +180,24 @@ token_type	*equation;
 int		*np, loc, len, level;
 {
 	token_type	*p1;
-	int		modified, single_modified;
+	int		modified = false, single_modified = false;
 	int		i, j, k;
-	long		v, v1, last_v;
+	long		v = 0, v1, last_v;
 	int		len_first;
 	int		loc1;
-	int		loc2, len2;
+	int		loc2, len2 = 0;
 	int		loct, lent;
 	int		count;
 	jmp_buf		save_save;
-	int		div_flag;
+	int		div_flag = 3;
 	int		vc, cnt;
 	sort_type	va[MAX_VARS];
 	double		d;
-	int		three = 3;
 
 	debug_string(3, "Entering pf_sub().");
-	modified = false;
-	single_modified = false;
 	loc1 = loc;
-	len2 = 0;
-	v = 0L;
-	div_flag = 2;
 	find_greatest_power(&equation[loc1], len, &v, &d, &j, &k, &div_flag);
-	if (v == 0L)
+	if (v == 0)
 		return false;
 	blt(save_save, jmp_save, sizeof(jmp_save));
 	if ((i = setjmp(jmp_save)) != 0) {
@@ -233,8 +218,7 @@ int		*np, loc, len, level;
 		for (vc = 0; vc < ARR_CNT(va); vc++) {
 			v1 = -1;
 			for (i = 0; i < n_trhs; i += 2) {
-				if (trhs[i].kind == VARIABLE
-				    && trhs[i].token.variable > last_v) {
+				if (trhs[i].kind == VARIABLE && trhs[i].token.variable > last_v) {
 					if (v1 == -1 || trhs[i].token.variable < v1) {
 						v1 = trhs[i].token.variable;
 						cnt = 1;
@@ -253,14 +237,14 @@ int		*np, loc, len, level;
 		cnt = -1;
 		if (v) {
 			if (!poly_in_v(trhs, n_trhs, v, true)) {
-				v = 0L;
+				v = 0;
 			}
 		}
 		for (i = 0; i < vc; i++) {
 			if ((va[i].v & VAR_MASK) <= SIGN) {
 				continue;
 			}
-			if (v == 0L) {
+			if (v == 0) {
 				if (poly_in_v(trhs, n_trhs, va[i].v, true)) {
 					v = va[i].v;
 				}
@@ -269,7 +253,7 @@ int		*np, loc, len, level;
 				cnt = va[i].count;
 			}
 		}
-		if (cnt <= 1 || v == 0L)
+		if (cnt <= 1 || v == 0)
 			goto skip_factor;
 		blt(tlhs, trhs, n_trhs * sizeof(token_type));
 		n_tlhs = n_trhs;
@@ -278,7 +262,7 @@ int		*np, loc, len, level;
 		}
 #if	!SILENT
 		if (debug_level >= 3) {
-			list_var(v, false);
+			list_var(v, 0);
 			printf(_("Differentiation successful using variable (%s).\n"), var_str);
 		}
 #endif
@@ -317,7 +301,7 @@ int		*np, loc, len, level;
 	}
 /* Now try and factor polynomials with symbolic factors. */
 	if (!modified) {
-		last_v = 0L;
+		last_v = 0;
 next_v:
 		p1 = &equation[loc];
 		blt(trhs, p1, len * sizeof(token_type));
@@ -326,8 +310,7 @@ next_v:
 		for (;;) {
 			v = -1;
 			for (i = 0; i < len; i += 2) {
-				if (p1[i].kind == VARIABLE
-				    && p1[i].token.variable > last_v) {
+				if (p1[i].kind == VARIABLE && p1[i].token.variable > last_v) {
 					if (v == -1 || p1[i].token.variable < v) {
 						v = p1[i].token.variable;
 					}
@@ -337,13 +320,13 @@ next_v:
 				break;
 			}
 			last_v = v;
-			if (find_greatest_power(trhs, n_trhs, &v, &d, &j, &k, &three) <= 1) {
+			if (find_greatest_power(trhs, n_trhs, &v, &d, &j, &k, &div_flag) <= 1) {
 				continue;
 			}
 			blt(tlhs, trhs, n_trhs * sizeof(token_type));
 			n_tlhs = n_trhs;
 			factorv(tlhs, &n_tlhs, v);
-			if (find_greatest_power(tlhs, n_tlhs, &v, &d, &j, &k, &three) != 1) {
+			if (find_greatest_power(tlhs, n_tlhs, &v, &d, &j, &k, &div_flag) != 1) {
 				continue;
 			}
 			blt(tlhs, &tlhs[j], k * sizeof(token_type));
@@ -422,16 +405,16 @@ int		*np, loc1, len, level;
 
 /*
  * Remove level 1 trivial factors and divides from "tlhs".
- * Return true if result is (or will be after unfactoring)
- * a level 1 plus type expression.
+ *
+ * Return true if result is a level 1 plus type expression.
  * If this returns false, nothing is removed.
  */
 int
 remove_factors()
 {
 	int	i, j, k;
-	int	plus_flag;
-	int	divide_flag;
+	int	plus_flag = false;
+	int	divide_flag = false;
 	int	op;
 
 	debug_string(3, "Entering remove_factors() with:");
@@ -439,9 +422,7 @@ remove_factors()
 	do {
 		simp_ssub(tlhs, &n_tlhs, 0L, 1.0, false, true, 0);
 	} while (uf_power(tlhs, &n_tlhs));
-	plus_flag = false;
-	divide_flag = false;
-	for (i = 0, j = 0, k = 0;; i++) {
+	for (i = 1, j = 0, k = 0;; i += 2) {
 		if (i >= n_tlhs) {
 			if (plus_flag && !divide_flag) {
 				if (k > 0)
@@ -456,37 +437,45 @@ remove_factors()
 			side_debug(3, tlhs, n_tlhs);
 			return true;
 		}
-		if (tlhs[i].kind == OPERATOR) {
-			if (tlhs[i].level == 1) {
-				op = tlhs[i].token.operatr;
-				if (op == PLUS || op == MINUS) {
-					plus_flag = true;
-					continue;
-				}
-				if (op != TIMES && op != DIVIDE)
-					return false;
-				if (plus_flag && !divide_flag) {
-					if (k > 0)
-						j--;
-					blt(&scratch[k], &tlhs[j], (i - j) * sizeof(token_type));
-					k += i - j;
-				}
-				plus_flag = false;
-				divide_flag = (op == DIVIDE);
-				j = i + 1;
-			} else if (tlhs[i].level == 2) {
-				switch (tlhs[i].token.operatr) {
-				case PLUS:
-				case MINUS:
-					plus_flag = true;
-				}
+		op = tlhs[i].token.operatr;
+		switch (tlhs[i].level) {
+		case 1:
+			switch (op) {
+			case PLUS:
+			case MINUS:
+				plus_flag = true;
+				continue;
+			case TIMES:
+			case DIVIDE:
+				break;
+			default:
+				return false;
 			}
+			if (plus_flag && !divide_flag) {
+				if (k > 0)
+					j--;
+				blt(&scratch[k], &tlhs[j], (i - j) * sizeof(token_type));
+				k += i - j;
+			}
+			plus_flag = false;
+			divide_flag = (op == DIVIDE);
+			j = i + 1;
+			break;
+		case 2:
+			switch (op) {
+			case PLUS:
+			case MINUS:
+				plus_flag = true;
+			}
+			break;
 		}
 	}
 }
 
 /*
  * This is the Euclidean GCD algorithm applied to polynomials.
+ * It needs to be made multi-variate.
+ *
  * Return number of iterations, if successful.
  * Return 0 on failure.
  */
@@ -498,12 +487,15 @@ long		*vp;
 	int	count;
 
 	for (count = 1;; count++) {
-		if (!poly_div(trhs, n_trhs, gcd_divisor, len_d, vp)) {
+		switch (poly_div(trhs, n_trhs, gcd_divisor, len_d, vp)) {
+		case 0:
+			/* divide failed */
 			return 0;
+		case 2:
+			/* Total success!  Remainder is zero. */
+			return count;
 		}
-		if (n_trhs == 1 && trhs[0].kind == CONSTANT && trhs[0].token.constant == 0.0) {
-			break;
-		}
+		/* swap trhs[] (remainder) and gcd_divisor[] */
 		if (len_d > n_tokens || n_trhs > DIVISOR_SIZE)
 			return 0;
 		blt(scratch, trhs, n_trhs * sizeof(token_type));
@@ -513,11 +505,11 @@ long		*vp;
 		n_trhs = len_d;
 		len_d = i;
 	}
-	return count;
 }
 
 /*
  * Compute polynomial Greatest Common Divisor of "larger" and "smaller".
+ *
  * Return true if successful.
  * Return the GCD in "trhs".
  * Return "larger"/GCD in "tlhs".
@@ -557,9 +549,8 @@ long		v;
 			return 0;
 		blt(gcd_divisor, tlhs, n_tlhs * sizeof(token_type));
 		len_d = n_tlhs;
-		if (!poly_div(larger, llen, gcd_divisor, len_d, &v)
-		    || n_trhs != 1 || trhs[0].kind != CONSTANT || trhs[0].token.constant != 0.0) {
-			error("Polynomial GCD found, but larger divide failed.");
+		if (poly_div(larger, llen, gcd_divisor, len_d, &v) != 2) {
+			debug_string(0, "Polynomial GCD found, but larger divide failed.");
 			return 0;
 		}
 	}
@@ -575,6 +566,7 @@ long		v;
 
 /*
  * Compute polynomial Greatest Common Divisor of "larger" and "smaller".
+ *
  * Return true if successful.
  * Return "larger"/GCD in "tlhs".
  * Return "smaller"/GCD in "trhs".
@@ -610,9 +602,8 @@ long		v;
 	if (count > 1) {
 		if (!level1_plus(gcd_divisor, len_d))
 			return 0;
-		if (!poly_div(smaller, slen, gcd_divisor, len_d, &v)
-		    || n_trhs != 1 || trhs[0].kind != CONSTANT || trhs[0].token.constant != 0.0) {
-			error("Polynomial GCD found, but smaller divide failed.");
+		if (poly_div(smaller, slen, gcd_divisor, len_d, &v) != 2) {
+			debug_string(0, "Polynomial GCD found, but smaller divide failed.");
 			return 0;
 		}
 		blt(trhs, gcd_divisor, len_d * sizeof(token_type));
@@ -623,18 +614,15 @@ long		v;
 		len_d = n_tlhs;
 		blt(tlhs, trhs, n_trhs * sizeof(token_type));
 		n_tlhs = n_trhs;
-		if (!poly_div(larger, llen, tlhs, n_tlhs, &v)
-		    || n_trhs != 1 || trhs[0].kind != CONSTANT || trhs[0].token.constant != 0.0) {
-			error("Polynomial GCD found, but larger divide failed.");
+		if (poly_div(larger, llen, tlhs, n_tlhs, &v) != 2) {
+			debug_string(0, "Polynomial GCD found, but larger divide failed.");
 			return 0;
 		}
 		blt(trhs, gcd_divisor, len_d * sizeof(token_type));
 		n_trhs = len_d;
 	} else {
 		n_trhs = 1;
-		trhs[0].level = 1;
-		trhs[0].kind = CONSTANT;
-		trhs[0].token.constant = 1.0;
+		trhs[0] = one_token;
 	}
 	uf_simp(tlhs, &n_tlhs);
 	uf_simp(trhs, &n_trhs);
@@ -649,8 +637,8 @@ long		v;
  */
 int
 is_integer_expr(equation, n)
-token_type	*equation;
-int		n;
+token_type	*equation;	/* expression pointer */
+int		n;		/* length of expression */
 {
 	int	i2;
 	double	d;
@@ -687,7 +675,7 @@ mod_recurse(equation, np, loc, level)
 token_type	*equation;
 int		*np, loc, level;
 {
-	int	modified;
+	int	modified = false;
 	int	i, j, k;
 	int	i1, i2, i3, i4, i5;
 	int	op;
@@ -696,7 +684,6 @@ int		*np, loc, level;
 	long	v;
 	int	diff_sign;
 
-	modified = false;
 	for (i = loc; i < *np && equation[i].level >= level;) {
 		if (equation[i].level > level) {
 			modified |= mod_recurse(equation, np, i, level + 1);
@@ -853,7 +840,7 @@ int		*np, loc, level;
  * and denominator by the GCD.
  */
 int
-polydiv_simp(equation, np)
+poly_gcd_simp(equation, np)
 token_type	*equation;
 int		*np;
 {
@@ -865,12 +852,11 @@ polydiv_recurse(equation, np, loc, level)
 token_type	*equation;
 int		*np, loc, level;
 {
-	int	modified;
+	int	modified = false;
 	int	i, j, k;
 	int	last_op2;
 	int	len1, len2;
 
-	modified = false;
 	for (i = loc; i < *np && equation[i].level >= level;) {
 		if (equation[i].level > level) {
 			modified |= polydiv_recurse(equation, np, i, level + 1);
@@ -967,19 +953,15 @@ pdiv_recurse(equation, np, loc, level, code)
 token_type	*equation;
 int		*np, loc, level, code;
 {
-	int	modified;
+	int	modified = false;
 	int	i, j, k;
 	int	op, op2;
 	int	last_op2;
-	int	len1, len2;
-	int	real_len1;
+	int	len1, len2, real_len1;
 	long	v;
 	int	rv;
-	int	flag;
-	int	power_flag;
-	int	rem_zero;
+	int	flag, power_flag;
 
-	modified = false;
 	for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
 		if (equation[i].level == level && equation[i].token.operatr == DIVIDE) {
 			for (k = i + 2;; k += 2) {
@@ -1045,9 +1027,7 @@ next_thingy:
 				} else {
 					rv = smart_div(&equation[j], len2, &equation[i+1], len1);
 				}
-				rem_zero = (rv > 0 && n_trhs == 1 && trhs[0].kind == CONSTANT
-				    && trhs[0].token.constant == 0.0);
-				if (power_flag && !rem_zero) {
+				if (power_flag && !(rv > 0 && REMAINDER_IS_ZERO())) {
 					rv = 0;
 				}
 				if (rv > 0 && (n_tlhs + 2 + n_trhs + len1) <= n_tokens) {
@@ -1150,7 +1130,8 @@ next_thingy:
 
 /*
  * Do polynomial division.
- * Returns true if successful.
+ *
+ * Returns non-zero if successful, 2 if remainder is 0.
  * Quotient is returned in "tlhs" and remainder in "trhs".
  *
  * If "*vp" is 0, automatically select the best base variable,
@@ -1225,9 +1206,10 @@ long		*vp;
 			return false;
 	}
 #if	!SILENT
+	/* Display debugging info. */
 	if (debug_level >= 3) {
-		list_var(*vp, false);
-		printf(_("poly_div starts using variable (%s):\n"), var_str);
+		list_var(*vp, 0);
+		printf(_("poly_div() starts using variable (%s):\n"), var_str);
 		side_debug(3, trhs, n_trhs);
 		side_debug(3, tlhs, n_tlhs);
 	}
@@ -1246,9 +1228,8 @@ long		*vp;
 	}
 	/* Initialize the quotient. */
 	n_quotient = 1;
-	quotient[0].level = 1;
-	quotient[0].kind = CONSTANT;
-	quotient[0].token.constant = 0.0;
+	quotient[0] = zero_token;
+	/* Store the divisor. */
 	if (n_tlhs > ARR_CNT(divisor))
 		return false;
 	blt(divisor, tlhs, n_tlhs * sizeof(token_type));
@@ -1256,9 +1237,10 @@ long		*vp;
 	sum_size = n_trhs + n_quotient;
 	/* Loop until polynomial division is finished. */
 	for (;;) {
-		sign = PLUS;
 		if (t1 > 0 && trhs[t1-1].token.operatr == MINUS)
 			sign = MINUS;
+		else
+			sign = PLUS;
 		if (t2 > 0 && divisor[t2-1].token.operatr == MINUS) {
 			if (sign == MINUS)
 				sign = PLUS;
@@ -1296,9 +1278,7 @@ long		*vp;
 			return false;
 		blt(&trhs[t1+1], &trhs[t1+len_t1], (n_trhs - (t1 + len_t1)) * sizeof(token_type));
 		n_trhs -= len_t1 - 1;
-		trhs[t1].level = 1;
-		trhs[t1].kind = CONSTANT;
-		trhs[t1].token.constant = 0.0;
+		trhs[t1] = zero_token;
 		for (i = 0; i < n_trhs; i++)
 			trhs[i].level++;
 		trhs[n_trhs].level = 1;
@@ -1320,9 +1300,7 @@ long		*vp;
 		i = n_trhs;
 		blt(&trhs[n_trhs], divisor, t2 * sizeof(token_type));
 		n_trhs += t2;
-		trhs[n_trhs].level = 1;
-		trhs[n_trhs].kind = CONSTANT;
-		trhs[n_trhs].token.constant = 0.0;
+		trhs[n_trhs] = zero_token;
 		n_trhs++;
 		blt(&trhs[n_trhs], &divisor[t2+len_t2], (n_divisor - (t2 + len_t2)) * sizeof(token_type));
 		n_trhs += (n_divisor - (t2 + len_t2));
@@ -1339,11 +1317,10 @@ long		*vp;
 			n_tlhs = n_quotient;
 			side_debug(3, tlhs, n_tlhs);
 			side_debug(3, trhs, n_trhs);
-			if (n_trhs == 1 && trhs[0].kind == CONSTANT && trhs[0].token.constant == 0.0)
-				return true;
+			if (REMAINDER_IS_ZERO())
+				return 2;
 			if ((n_trhs + n_quotient) >= (sum_size /* - (sum_size / 10) */)) {
-				if ((n_trhs + 1) > sum_size
-				    && n_trhs > n_divisor)
+				if ((n_trhs + 1) > sum_size && n_trhs > n_divisor)
 					return -2;
 				else
 					return -1;
@@ -1385,22 +1362,20 @@ int		len1;		/* length of dividend */
 token_type	*d2;		/* pointer to divisor */
 int		len2;		/* length of divisor */
 {
-#define	MAX_TRIES	100
-
 	int		i, j, k;
 	int		t1, len_t1;
-	int		t2, len_t2;
+	int		t2, len_t2 = 0;
 	int		sign;
 	int		old_n_quotient;
 	int		trhs_size;
 	int		term_size, term_count;
-	int		term_pos, skip_terms[MAX_TRIES];
+	int		term_pos, skip_terms[100];
 	int		skip_count;
 	token_type	*qp;
 	int		q_size;
 	int		sum_size;
 	int		count;
-	int		dcount;		/* divisor term count */
+	int		dcount = 0;		/* divisor term count */
 	int		flag;
 
 	blt(trhs, d1, len1 * sizeof(token_type));
@@ -1410,12 +1385,12 @@ int		len2;		/* length of divisor */
 	/* Do the basic unfactoring and simplification of the dividend and divisor. */
 	uf_simp_no_repeat(trhs, &n_trhs);
 	uf_simp_no_repeat(tlhs, &n_tlhs);
+	/* Display debugging info. */
 	debug_string(3, "smart_div() starts:");
 	side_debug(3, trhs, n_trhs);
 	side_debug(3, tlhs, n_tlhs);
 	/* Find which divisor term to use. */
-	dcount = 0;
-	for (i = 0, j = 0, k = 0, len_t2 = 0, flag = false;; i++) {
+	for (i = 0, j = 0, k = 0, flag = false;; i++) {
 		if (i >= n_tlhs || (tlhs[i].kind == OPERATOR && tlhs[i].level == 1
 		    && (tlhs[i].token.operatr == PLUS || tlhs[i].token.operatr == MINUS))) {
 			dcount++;
@@ -1428,8 +1403,7 @@ int		len2;		/* length of divisor */
 			}
 			flag = false;
 			j = i + 1;
-		} else if (tlhs[i].kind == VARIABLE
-		    && (tlhs[i].token.variable & VAR_MASK) > SIGN) {
+		} else if (tlhs[i].kind == VARIABLE && (tlhs[i].token.variable & VAR_MASK) > SIGN) {
 			flag = true;
 		}
 		if (i >= n_tlhs)
@@ -1439,17 +1413,14 @@ int		len2;		/* length of divisor */
 		return false;
 	/* Initialize the quotient. */
 	n_quotient = 1;
-	quotient[0].level = 1;
-	quotient[0].kind = CONSTANT;
-	quotient[0].token.constant = 0.0;
+	quotient[0] = zero_token;
 	if (n_tlhs > ARR_CNT(divisor))
 		return false;
 	blt(divisor, tlhs, n_tlhs * sizeof(token_type));
 	n_divisor = n_tlhs;
 try_one:
 	trhs_size = n_trhs;
-	skip_count = 0;
-	for (count = 0;;) {
+	for (skip_count = 0, count = 0;;) {
 		sum_size = n_trhs + n_quotient;
 		for (term_count = 1, q_size = 0;; term_count++) {
 			if (!get_term(trhs, n_trhs, term_count, &t1, &len_t1))
@@ -1499,8 +1470,7 @@ try_one:
 			}
 end_div:
 			if (dcount > 1) {
-				i = n_quotient + n_trhs;
-				if (i >= trhs_size + 1) {
+				if (n_quotient + n_trhs >= trhs_size + 1) {
 					return false;
 				}
 			}
@@ -1558,9 +1528,7 @@ end_div2:
 		n_tlhs = n_trhs;
 		blt(&trhs[t1+1], &trhs[t1+len_t1], (n_trhs - (t1 + len_t1)) * sizeof(token_type));
 		n_trhs -= len_t1 - 1;
-		trhs[t1].level = 1;
-		trhs[t1].kind = CONSTANT;
-		trhs[t1].token.constant = 0.0;
+		trhs[t1] = zero_token;
 		for (i = 0; i < n_trhs; i++)
 			trhs[i].level++;
 		trhs[n_trhs].level = 1;
@@ -1582,9 +1550,7 @@ end_div2:
 		i = n_trhs;
 		blt(&trhs[n_trhs], divisor, t2 * sizeof(token_type));
 		n_trhs += t2;
-		trhs[n_trhs].level = 1;
-		trhs[n_trhs].kind = CONSTANT;
-		trhs[n_trhs].token.constant = 0.0;
+		trhs[n_trhs] = zero_token;
 		n_trhs++;
 		blt(&trhs[n_trhs], &divisor[t2+len_t2], (n_divisor - (t2 + len_t2)) * sizeof(token_type));
 		n_trhs += (n_divisor - (t2 + len_t2));
@@ -1593,7 +1559,7 @@ end_div2:
 		side_debug(3, trhs, n_trhs);
 		uf_tsimp(trhs, &n_trhs);
 		side_debug(4, trhs, n_trhs);
-		if (n_trhs == 1 && trhs[0].kind == CONSTANT && trhs[0].token.constant == 0.0)
+		if (REMAINDER_IS_ZERO())
 			goto end_div2;
 		if (dcount > 1) {
 			if ((n_trhs + n_quotient) >= sum_size) {
@@ -1635,10 +1601,9 @@ int		len;
 	int	i, j;
 	int	level;
 	int	rv;
-	int	constant_flag;
+	int	constant_flag = true;
 
 	rv = len;
-	constant_flag = true;
 	level = min_level(p1, len);
 	for (i = 0, j = -1; i < len; i++) {
 		if (p1[i].kind == OPERATOR) {
@@ -1763,13 +1728,12 @@ token_type	*p1;
 int		n1, loc;
 {
 	int	i, j, k;
-	int	divide_flag;
+	int	divide_flag = false;
 	int	level, div_level;
 	double	d, sub_count, sub_sum;
 
 	for (i = 0; i < VALUE_CNT; i++)
 		dp[i] = 0.0;
-	divide_flag = false;
 	for (i = loc; i < n1; i++) {
 		level = p1[i].level;
 		if (p1[i].kind == VARIABLE) {
@@ -1806,8 +1770,7 @@ int		n1, loc;
 		} else if (divide_flag && level <= div_level) {
 			divide_flag = false;
 		}
-		if (p1[j].token.operatr == POWER && level == p1[j+1].level
-		    && p1[j+1].kind == CONSTANT) {
+		if (p1[j].token.operatr == POWER && level == p1[j+1].level && p1[j+1].kind == CONSTANT) {
 			d = p1[j+1].token.constant - 1.0;
 			sub_count = 0.0;
 			sub_sum = 0.0;
@@ -1843,28 +1806,24 @@ double		*pp1;
 int		*tp1, *lentp1;
 int		*dcodep;	/* divide flag pointer indicates if term is a denominator */
 				/* for example: for x^5 it is false, for 1/x^5 it is true */
+				/* if equal to 3, ignore */
 {
 	int		i, j, k;
 	int		ii;
-	int		flag;
 	double		d;
-	int		divide_flag;
+	int		flag, divide_flag = false;
 	int		div_level;
 	int		level;
-	long		v;
-	int		was_power;
+	long		v = 0;
+	int		was_power = false;
 	double		last_va[VALUE_CNT];
 	double		va[VALUE_CNT];
 	int		rv;
-	int		count;
+	int		count = 0;
 
 	*pp1 = 0.0;
 	*tp1 = -1;
-	v = 0;
-	was_power = false;
 	rv = *dcodep;
-	count = 0;
-	divide_flag = false;
 	for (j = 0, i = 1;; i += 2) {
 		if (i >= n1 || ((p1[i].token.operatr == PLUS || p1[i].token.operatr == MINUS) && p1[i].level == 1)) {
 			divide_flag = false;
