@@ -17,7 +17,7 @@ int		sum_cmd(), product_cmd(), integrate_cmd(), nintegrate_cmd(), laplace_cmd();
 
 /* various functions that don't return int */
 char		*skip_space(), *skip_param();
-char		*getstring();
+char		*get_string();
 char		*parse_equation(), *parse_section(), *parse_var(), *parse_var2();
 char		*list_expression(), *list_equation();
 double		gcd(), my_round(), multiply_out_unique();
