@@ -39,7 +39,7 @@ int	have;	/* equation number to solve */
 		printf(_("Solve failed.\n"));
 	}
 #endif
-	return rv;
+	return(rv > 0);
 }
 
 /*
@@ -49,17 +49,21 @@ int	have;	/* equation number to solve */
  * to the LHS (via transposition), then moving everything not containing the variable to the
  * RHS.  Many tricks are used, and this routine works very well.
  *
- * Returns true if successful, with the result placed in the passed LHS and RHS.
- * Returns 2 if a solution was zero and removed.
- * Returns false on failure.
+ * "tlhs" and "trhs" are used by this routine.
+ *
+ * Returns a positive integer if successful, with the result placed in the passed LHS and RHS.
+ * Returns 1 for normal success.
+ * Returns 2 if successful and a solution was zero and removed.
+ * Returns 0 on failure.
+ * Returns -1 if the equation is an identity.
  */
 int
 solve_sub(wantp, wantn, leftp, leftnp, rightp, rightnp)
 token_type	*wantp;		/* expression to solve for */
 int		wantn;		/* size of expression to solve for (currently should always be 1) */
-token_type	*leftp;		/* LHS */
+token_type	*leftp;		/* LHS of equation */
 int		*leftnp;	/* size of LHS */
-token_type	*rightp;	/* RHS */
+token_type	*rightp;	/* RHS of equation */
 int		*rightnp;	/* size of RHS */
 {
 	int		i, j;
@@ -67,24 +71,20 @@ int		*rightnp;	/* size of RHS */
 	int		worked;
 	int		diff_sign;
 	int		op, op_kind;
-	token_type	*p1, *b1, *ep, want0;
+	token_type	*p1, *b1, *ep;
 	long		v;
 	int		need_flip;
-	int		uf_flag;
-	int		qtries;
+	int		uf_flag = false;
+	int		qtries = 0;
 	int		zflag;
 	int		zsolve;
-	int		inc_count;
-	int		zero_solved;
+	int		inc_count = 0;
+	int		zero_solved = false;
 	double		numerator, denominator;
-	int		success;
+	int		success = 1;
 
-	success = 1;
 	repeat_count = 0;
-	qtries = 0;
-	zero_solved = false;
-	uf_flag = false;
-	inc_count = 0;
+	/* copy the equation to temporary storage where it will be manipulated */
 	n_tlhs = *leftnp;
 	blt(tlhs, leftp, n_tlhs * sizeof(*leftp));
 	n_trhs = *rightnp;
@@ -94,7 +94,7 @@ int		*rightnp;	/* size of RHS */
 		return false;
 	}
 	if (n_tlhs <= 0 || n_trhs <= 0) {
-		error(_("Please enter an equation."));
+		error(_("Please enter an equation or type \"help\"."));
 		return false;
 	}
 	if (wantp->kind == VARIABLE) {
@@ -170,11 +170,15 @@ no_simp:
 						}
 					}
 				}
-				if (op == TIMES || op == DIVIDE || op == POWER) {
+				switch (op) {
+				case TIMES:
+				case DIVIDE:
+				case POWER:
 					b1 = trhs;
 					op = PLUS;
 					for (p1 = b1; p1 < ep; p1++)
 						p1->level++;
+					break;
 				}
 				if (!g_of_f(op, b1, trhs, &n_trhs, tlhs, &n_tlhs))
 					return false;
@@ -281,8 +285,7 @@ zero_simp:
 		if (p1 >= ep || (p1->level == 1 && p1->kind == OPERATOR)) {
 			if (!found) {
 				if ((p1 < ep || found_count || zsolve || n_tlhs > 1 || tlhs[0].kind != CONSTANT)
-				    && (p1 - b1 != 1 || b1->kind != CONSTANT
-				    || b1->token.constant != 1.0
+				    && (p1 - b1 != 1 || b1->kind != CONSTANT || b1->token.constant != 1.0
 				    || p1 >= ep || p1->token.operatr != DIVIDE)) {
 					if (op == 0) {
 						for (;; p1++) {
@@ -358,7 +361,7 @@ zero_simp:
 #if	!SILENT
 						printf(_("That is, the left equation side is identical to the right equation side.\n"));
 #endif
-						return false;
+						return -1;
 					}
 					found = false;
 					for (i = 0; i < n_tlhs; i += 2) {
@@ -507,7 +510,7 @@ zero_simp:
 					zero_solved = false;
 					qtries = 0;
 					if (!increase(b1->token.constant, v)) {
-						error(_("increase() function failed."));
+						error(_("Failed to raise both sides to a power."));
 						return false;
 					}
 					uf_flag = true;
@@ -521,10 +524,7 @@ zero_simp:
 				blt(leftp, tlhs, n_tlhs * sizeof(*leftp));
 				*rightnp = n_trhs;
 				blt(rightp, trhs, n_trhs * sizeof(*rightp));
-				want0.level = 1;
-				want0.kind = CONSTANT;
-				want0.token.constant = 0.0;
-				if (!solve_sub(&want0, 1, leftp, leftnp, rightp, rightnp))
+				if (solve_sub(&zero_token, 1, leftp, leftnp, rightp, rightnp) <= 0)
 					return false;
 				if (zero_solved) {
 					qtries++;
@@ -595,8 +595,7 @@ lonely:
 					b1++;
 					break;
 				}
-				if (b1->kind == VARIABLE
-				    && b1->token.variable == v) {
+				if (b1->kind == VARIABLE && b1->token.variable == v) {
 					flag = true;
 				}
 				if (b1 == tlhs)
@@ -623,8 +622,7 @@ lonely:
 					b1++;
 					break;
 				}
-				if (b1->kind == VARIABLE
-				    && b1->token.variable == v) {
+				if (b1->kind == VARIABLE && b1->token.variable == v) {
 					flag = true;
 				}
 				if (b1 == tlhs)
@@ -642,8 +640,7 @@ lonely:
 	}
 	b1 = p1 + 1;
 	op = p1->token.operatr;
-	if (op == POWER && b1->level == 1
-	    && b1->kind == CONSTANT && b1->token.constant == d) {
+	if (op == POWER && b1->level == 1 && b1->kind == CONSTANT && b1->token.constant == d) {
 		return(g_of_f(POWER, b1, tlhs, &n_tlhs, trhs, &n_trhs));
 	}
 	if (!foundp) {
@@ -704,7 +701,7 @@ long	v;
 	int		len;
 	int		alen, blen;
 	int		aloc;
-	double		high_power;
+	double		high_power = 0.0;
 
 	uf_simp(trhs, &n_trhs);
 	factorv(trhs, &n_trhs, v);
@@ -712,7 +709,6 @@ long	v;
 
 	found = false;
 	op = 0;
-	high_power = 0.0;
 	ep = &trhs[n_trhs];
 	for (x1tp = p1 = trhs;; p1++) {
 		if (p1 >= ep || (p1->level == 1 && p1->kind == OPERATOR)) {
@@ -756,21 +752,15 @@ long	v;
 						}
 					}
 				}
-			} else if (op == POWER) {
-				if (found && (p1 - 1)->level == 2
-				    && (p1 - 1)->kind == CONSTANT
-				    && (p1 - 1)->token.constant > high_power) {
-					high_power = (p1 - 1)->token.constant;
-					a1p = x1p = x1tp;
-					a2p = p1;
-					a2ep = a2p;
-				}
+			} else if (op == POWER && found && (p1 - 1)->level == 2
+			    && (p1 - 1)->kind == CONSTANT && (p1 - 1)->token.constant > high_power) {
+				high_power = (p1 - 1)->token.constant;
+				a1p = x1p = x1tp;
+				a2p = p1;
+				a2ep = a2p;
 			}
 			if (p1 >= ep) {
-				if (high_power == 0.0)
-					return false;
-				else
-					break;
+				break;
 			}
 		}
 		if (p1->level == 1) {
@@ -784,12 +774,13 @@ long	v;
 				if (p1->level == 2) {
 					op = p1->token.operatr;
 				}
-			} else if (op == 0) {
-				if (p1->kind == VARIABLE && p1->token.variable == v)
-					found = true;
+			} else if (op == 0 && p1->kind == VARIABLE && p1->token.variable == v) {
+				found = true;
 			}
 		}
 	}
+	if (high_power == 0.0)
+		return false;
 	if (a1p > trhs && (a1p - 1)->token.operatr == MINUS)
 		opx1 = MINUS;
 	else
@@ -830,11 +821,9 @@ long	v;
 						if (p2 >= p1)
 							break;
 					}
-					if (p2->level == 2) {
-						if (p2->kind == OPERATOR) {
-							x2p = p2 + 1;
-							op2 = p2->token.operatr;
-						}
+					if (p2->level == 2 && p2->kind == OPERATOR) {
+						x2p = p2 + 1;
+						op2 = p2->token.operatr;
 					}
 				}
 			}
@@ -848,10 +837,8 @@ long	v;
 				x2p = p1 + 1;
 			}
 		} else {
-			if (p1->kind == OPERATOR) {
-				if (p1->level == 2) {
-					op = p1->token.operatr;
-				}
+			if (p1->kind == OPERATOR && p1->level == 2) {
+				op = p1->token.operatr;
 			}
 		}
 	}
@@ -870,7 +857,7 @@ big_bbreak:
 	default:
 		return false;
 	}
-	blt(scratch, b1p, (int) ((char *) x2p - (char *) b1p));
+	blt(scratch, b1p, (char *) x2p - (char *) b1p);
 	len = x2p - b1p;
 	scratch[len].level = 7;
 	scratch[len].kind = CONSTANT;
@@ -879,7 +866,7 @@ big_bbreak:
 	else
 		scratch[len].token.constant = 1.0;
 	len++;
-	blt(&scratch[len], b2p, (int) ((char *) b2ep - (char *) b2p));
+	blt(&scratch[len], b2p, (char *) b2ep - (char *) b2p);
 	len += (b2ep - b2p);
 	blen = len;
 	j = min_level(scratch, len);
@@ -907,7 +894,7 @@ big_bbreak:
 	scratch[len].token.operatr = TIMES;
 	len++;
 	aloc = len;
-	blt(&scratch[len], a1p, (int) ((char *) x1p - (char *) a1p));
+	blt(&scratch[len], a1p, (char *) x1p - (char *) a1p);
 	len += (x1p - a1p);
 	scratch[len].level = 7;
 	scratch[len].kind = CONSTANT;
@@ -916,7 +903,7 @@ big_bbreak:
 	else
 		scratch[len].token.constant = 1.0;
 	len++;
-	blt(&scratch[len], a2p, (int) ((char *) a2ep - (char *) a2p));
+	blt(&scratch[len], a2p, (char *) a2ep - (char *) a2p);
 	len += (a2ep - a2p);
 	alen = len - aloc;
 	j = min_level(&scratch[aloc], len - aloc);
@@ -928,9 +915,7 @@ big_bbreak:
 	scratch[len].token.operatr = TIMES;
 	len++;
 	k = len;
-	scratch[len].level = 1;
-	scratch[len].kind = CONSTANT;
-	scratch[len].token.constant = 0.0;
+	scratch[len] = zero_token;
 	len++;
 	for (p2 = p1 = trhs;; p1++) {
 		if (p1 >= ep || (p1->level == 1 && p1->kind == OPERATOR)) {
@@ -941,7 +926,7 @@ big_bbreak:
 					scratch[len].token.operatr = PLUS;
 					len++;
 				}
-				blt(&scratch[len], p2, (int) ((char *) p1 - (char *) p2));
+				blt(&scratch[len], p2, (char *) p1 - (char *) p2);
 				len += (p1 - p2);
 			}
 			if (p1 >= ep)
@@ -1084,27 +1069,13 @@ int		*side2np;	/* pointer to the length of "side2p" */
 				op = PLUS;
 				continue;
 			case MODULUS:
-				if (op != MODULUS) {
-					printf(_("Something is wrong with the solve routine!\n"));
-				} else {
-					operandp = p1 + 1;
-				}
+				operandp = p1 + 1;
 				continue;
 			}
 			break;
 		}
 	}
 	operandn = p1 - operandp;
-	if (op == MODULUS) {
-		if (*side2np == 1 && side2p->kind == CONSTANT) {
-			if (side2p->token.constant < 0.0
-			    || (operandn == 1 && operandp->kind == CONSTANT
-			    && side2p->token.constant >= fabs(operandp->token.constant))) {
-				error(_("There are no solutions."));
-				return false;
-			}
-		}
-	}
 	if (op == POWER && operandp == side1p) {
 		if (!get_constant(side2p, *side2np, &d1))
 			return false;
@@ -1163,8 +1134,7 @@ int		*side2np;	/* pointer to the length of "side2p" */
 		}
 	}
 #endif
-	if (*side1np + operandn + 3 > n_tokens
-	    || *side2np + operandn + 5 > n_tokens) {
+	if (*side1np + operandn + 3 > n_tokens || *side2np + operandn + 5 > n_tokens) {
 		error_huge();
 	}
 	for (p2 = side1p; p2 < ep; p2++)
@@ -1187,7 +1157,7 @@ int		*side2np;	/* pointer to the length of "side2p" */
 		p2->kind = OPERATOR;
 		p2->token.operatr = TIMES;
 		p2++;
-		blt(p2, operandp, (int) ((char *) p1 - (char *) operandp));
+		blt(p2, operandp, (char *) p1 - (char *) operandp);
 		*side1np += 3 + operandn;
 		break;
 	case POWER:
@@ -1203,7 +1173,7 @@ int		*side2np;	/* pointer to the length of "side2p" */
 		p2->kind = OPERATOR;
 		p2->token.operatr = DIVIDE;
 		p2++;
-		blt(p2, operandp, (int) ((char *) p1 - (char *) operandp));
+		blt(p2, operandp, (char *) p1 - (char *) operandp);
 		*side1np += 3 + operandn;
 		break;
 	case TIMES:
@@ -1211,7 +1181,7 @@ int		*side2np;	/* pointer to the length of "side2p" */
 		p2->kind = OPERATOR;
 		p2->token.operatr = DIVIDE;
 		p2++;
-		blt(p2, operandp, (int) ((char *) p1 - (char *) operandp));
+		blt(p2, operandp, (char *) p1 - (char *) operandp);
 		*side1np += 1 + operandn;
 		break;
 	case DIVIDE:
@@ -1219,7 +1189,7 @@ int		*side2np;	/* pointer to the length of "side2p" */
 		p2->kind = OPERATOR;
 		p2->token.operatr = TIMES;
 		p2++;
-		blt(p2, operandp, (int) ((char *) p1 - (char *) operandp));
+		blt(p2, operandp, (char *) p1 - (char *) operandp);
 		*side1np += 1 + operandn;
 		break;
 	case PLUS:
@@ -1227,7 +1197,7 @@ int		*side2np;	/* pointer to the length of "side2p" */
 		p2->kind = OPERATOR;
 		p2->token.operatr = MINUS;
 		p2++;
-		blt(p2, operandp, (int) ((char *) p1 - (char *) operandp));
+		blt(p2, operandp, (char *) p1 - (char *) operandp);
 		*side1np += 1 + operandn;
 		break;
 	case MINUS:
@@ -1235,11 +1205,11 @@ int		*side2np;	/* pointer to the length of "side2p" */
 		p2->kind = OPERATOR;
 		p2->token.operatr = PLUS;
 		p2++;
-		blt(p2, operandp, (int) ((char *) p1 - (char *) operandp));
+		blt(p2, operandp, (char *) p1 - (char *) operandp);
 		*side1np += 1 + operandn;
 		break;
 	}
-	blt(&side2p[*side2np], &side1p[oldn], (int) (*side1np - oldn) * sizeof(*side1p));
+	blt(&side2p[*side2np], &side1p[oldn], (*side1np - oldn) * sizeof(*side1p));
 	*side2np += *side1np - oldn;
 	if (op == POWER && operandn == 1 && operandp->kind == CONSTANT) {
 		f_to_fraction(operandp->token.constant, &numerator, &denominator);
@@ -1292,17 +1262,13 @@ int		*side2np;
 	blt(side2p + 2, side2p, *side2np * sizeof(*side2p));
 	*side2np += 2;
 
-	side1p->level = 1;
-	side1p->kind = CONSTANT;
-	side1p->token.constant = 1.0;
+	*side1p = one_token;
 	side1p++;
 	side1p->level = 1;
 	side1p->kind = OPERATOR;
 	side1p->token.operatr = DIVIDE;
 
-	side2p->level = 1;
-	side2p->kind = CONSTANT;
-	side2p->token.constant = 1.0;
+	*side2p = one_token;
 	side2p++;
 	side2p->level = 1;
 	side2p->kind = OPERATOR;
