@@ -44,12 +44,11 @@ sf_recurse(equation, np, loc, level, start_flag)
 token_type	*equation;
 int		*np, loc, level, start_flag;
 {
-	int	modified;
+	int	modified = false;
 	int	i, j, k;
 	int	op;
 	int	len1, len2;
 
-	modified = false;
 	if (!start_flag) {
 		for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
 			if (equation[i].level == level
@@ -108,10 +107,8 @@ int		*np, loc, i1, n1, i2, n2, level, check_flag;
 	int	len;
 	int	e1, e2;
 	int	op1, op2;
-	int	div_flag1, div_flag2;
+	int	div_flag1 = false, div_flag2 = false;
 
-	div_flag1 = false;
-	div_flag2 = false;
 	e1 = i1 + n1;
 	e2 = i2 + n2;
 	op2 = equation[i2-1].token.operatr;
@@ -291,7 +288,7 @@ int		*np, loc, level;
 	int		i;
 	int		len;
 	int		di, edi;
-	int		grouper;
+	int		grouper = false;
 	int		e1;
 
 	for (i = loc; i < *np && equation[i].level >= level;) {
@@ -307,31 +304,30 @@ int		*np, loc, level;
 	e1 = i;
 	di = -1;
 	edi = e1;
-	grouper = false;
 	for (i = loc + 1; i < e1; i += 2) {
 		if (equation[i].level == level) {
 			if (equation[i].token.operatr == DIVIDE) {
 				if (di < 0) {
 					di = i;
-				} else {
-					grouper = true;
-					for (len = i + 2; len < e1; len += 2) {
-						if (equation[len].level == level
-						    && equation[len].token.operatr != DIVIDE)
-							break;
-					}
-					len -= i;
-					if (edi == e1) {
-						i += len;
-						edi = i;
-						continue;
-					}
-					blt(scratch, &equation[i], len * sizeof(*equation));
-					blt(&equation[di+len], &equation[di], (i - di) * sizeof(*equation));
-					blt(&equation[di], scratch, len * sizeof(*equation));
-					edi += len;
-					i += len - 2;
+					continue;
 				}
+				grouper = true;
+				for (len = i + 2; len < e1; len += 2) {
+					if (equation[len].level == level
+					    && equation[len].token.operatr != DIVIDE)
+						break;
+				}
+				len -= i;
+				if (edi == e1) {
+					i += len;
+					edi = i;
+					continue;
+				}
+				blt(scratch, &equation[i], len * sizeof(*equation));
+				blt(&equation[di+len], &equation[di], (i - di) * sizeof(*equation));
+				blt(&equation[di], scratch, len * sizeof(*equation));
+				edi += len;
+				i += len - 2;
 			} else {
 				if (di >= 0 && edi == e1)
 					edi = i;
